// phpcs:ignoreFile
/**
 * @summary     TableCalc
 * @description Calculate in HTML tables
 * @version     2.0.5
 * @file        jquery.tableCalc.min.js
 * @author      Tommy Hansen
 * @copyright   Consertia AS, Copyright 2018-2025.
 * @license     When used with BulkTableEditor (license) no license needed
 */
(function ($, window) {

    "use strict";

    var calcRow  = {};
    var settings = {};
    var licM     = "<h2>PS! Your license is not valid or has expired, please renew!</h2>";
    var liv      = false;
    var methods  =
    {
        init: function(options) {
            
            options = $.extend( {}, $.fn.tableCalc.defaults, options );
            settings = options;
            settings.filterRows = {};

            return this.each(function() {
                var $this = $(this);
               
                //if plugin has not been initialized yet
                if (!settings.data){    
                   
                    //set data (element, elementId)
                    settings.data = $this.data($this[0].nodeName, $this[0].id);

                    //set event for when to calculate 
                    setEvent.call(this, options); 
                    
                    // calculate all rows if option is true
                    if(settings.calcOnLoad){
                        calculate(settings.calcType, settings.calcCustom, settings);
                    }

                }
            });
        },

        destroy: function() {
            $(window).off(".tableCalc");

            return this.each(function() {
                var $this = $(this);
                var data = $this.data("tableCalc");

                $this.removeData("tableCalc");
            });
        },

        //returns current row as JSON
        getCurrentRow: function () {
            //Give user a json object for saving values
            if (calcRow.obj) {
                return calcRow.obj;
            }
            return 0;
        },

        //calculates custom calculations
        calculate: function(str, custom){ 
            settings.data = this;
            return calculate(str, custom, settings);
        },

        getAllRows: function(){
            settings.data = this;
            return getAllRows(settings);
        },

        getSum: function(col){
            if(!liv){return 0;}
            settings.data = this;
            return getSum(settings, col);
        },

        getAverage: function(col){
            if(!liv){return 0;}
            settings.data = this;
            return getAverage(settings, col);
        },

        getMin: function(col){
            if(!liv){return 0;}
            settings.data = this;
            return getMin(settings, col);
        },

        getMax: function(col){
            if(!liv){return 0;}
            settings.data = this;
            return getMax(settings, col);
        },

        sortColumn: function(col, type){
            if(!liv){return 0;}
            settings.data = this;
            return sortColumn(settings, col, type);
        }
        
    };

    function getMin(options, col){
        var arr = getSumArray(options, col);
        //var result = Number(Math.min(...arr)).toFixed(options.decimals); //not able to minify
        var result = Number(arr.reduce(function(a, b){
            return Math.min(a, b);
        }));
        return result;
    }
    
    function getMax(options, col){   
        var arr = getSumArray(options, col);
        //var result = Number(Math.max(...arr)).toFixed(options.decimals); //not able to minify
        var result = Number(arr.reduce(function(a, b){
            return Math.max(a, b);
        }));
        return result;
    }
    
    function getSum(options, col){
        var arr = getSumArray(options, col);
        var result = 0;
        for(var i = 0; i < arr.length; i++){
            if(arr[i].length > 0){
                result += parseFloat(arr[i]);
            } 
        }
        return result.toFixed(options.decimals);
    }
    
    function getAverage(options, col){
        var arr = getSumArray(options, col);
        var tot = 0;
        var result = 0;
        var count = 0;
        for(var i = 0; i < arr.length; i++){
            if(arr[i].length > 0){
                tot += parseFloat(arr[i]);
                count++;
            } 
        }
        if(tot > 0){
            result = tot / count;
        }
        return result.toFixed(options.decimals);
    }
 
    function getAllRows(options){
        var $data = options.data;
        var rows = {};
        var arr = [];
        try{
            rows = $data[0].children[1].rows;//childNodes[3].rows; 
        }
        catch(e){
            console.log("No rows found: "+ e.message);
        }
        for(var i = 0; i < rows.length; i++){
            var rowdata = rows[i];
           //text columns
           if(options.textColumns.length > 0){
                for(var ii = 0; ii < options.textColumns.length; ii++){

                    //NOTE - if changes below - change also in function setEvent()

                    var valType = rowdata.cells[options.textColumns[ii]].children[0].type;
                    //check if we have a select box
                    if(rowdata.cells[options.textColumns[ii]].children[0].nodeName === "SELECT"){
                        valType = "select";
                    }
                    var v = "";
                    var r = rowdata.cells[options.textColumns[ii]].children.length;
                    switch(valType){
                        case "date":
                        case "datetime":
                        case "text":
                            v = rowdata.cells[options.textColumns[ii]].children[0].value;
                            break;
                        case "checkbox":
                            v = rowdata.cells[options.textColumns[ii]].children[0].checked;
                            break;
                        case "radio":
                            for(var i = 0; i < r; i++){
                                if(rowdata.cells[options.textColumns[ii]].children[i].nodeName === "INPUT"){
                                    if(rowdata.cells[options.textColumns[ii]].children[i].checked){
                                        v = rowdata.cells[options.textColumns[ii]].children[i].id;
                                        break;
                                    }
                                }
                            };
                            break;
                        case "select":
                            r = rowdata.cells[options.textColumns[ii]].children[0].children.length;
                            for(var i = 0; i < r; i++){
                                if(rowdata.cells[options.textColumns[ii]].children[0].children[i].selected){
                                    v = rowdata.cells[options.textColumns[ii]].children[0].children[i].value;
                                    break;
                                } 
                            }
                            break;
                        default:
                            try{
                                v = rowdata.cells[options.textColumns[ii]].children[0].value;
                            }
                            catch(e){
                                v = "Input type not supported ";
                            }
                            break;
                    }
                    //add json string
                    arr.push(JSON.stringify({"id": rows[i].children[options.rowId].id, "column": rows[i].children[options.textColumns[ii]].cellIndex, "value": v }));
                }
            }
            //calc columns
           if(options.calcColumns.length > 0){
               for(var ii = 0; ii < options.calcColumns.length; ii++){
                   //add json string
                   arr.push(JSON.stringify({"id": rows[i].children[options.rowId].id, "column": rows[i].children[options.calcColumns[ii]].cellIndex, "value": rows[i].children[options.calcColumns[ii]].children[0].value }));
               }
           }
           //sum column
           if(options.calcColumn_sum !== null){
                //add json string
                arr.push(JSON.stringify({"id": rows[i].children[options.rowId].id, "column": rows[i].children[options.calcColumn_sum].cellIndex, "value": rows[i].children[options.calcColumn_sum].children[0].value }));
           }   
        }
        return arr;
    }

    
    function getSumArray(options, col){
        if(options.calcColumn_sum !== null){
            var $data = options.data;
            var rows = {};
            var arr = [];
            
            try{
                rows = $data[0].children[1].rows;//childNodes[3].rows;
            }
            catch(e){
                console.log("No rows found: " + e.message);
            }
            
            for(var i = 0; i < rows.length; i++){
                if(col === undefined || col === null){
                    arr.push(rows[i].children[options.calcColumn_sum].children[0].value.replace(",", "."));
                }
                else{
                    //console.log("col:"+col+" row:"+i + " value: "+ rows[i].children[col].children[0].value);
                    arr.push(rows[i].children[col].children[0].value.replace(",", ".")); 
                }
            }
            return arr;
        }
    }

    //calculate custom calculations
    function calculate(str, custom, options){ 
        if(str.length > 0){
            options.calcType = str;
            if(str.toLowerCase() == "c" && custom.length > 0){
                options.calcCustom = custom;
            }

            //special calculations - to same or other column
            var outcolumn = "";
            var carr = options.calcCustom.split("=="); //(0:1 * 0.5)==0:1
            if(carr.length > 1){
                options.calcCustom = carr[0];
                var col   = carr[1].split(":");
                outcolumn = col[1].trim();
                
                //find out witch colums to calculate
                var expression = /0:[0-9]*/g;
                var matches, output = [];
                while (matches = expression.exec(options.calcCustom)) {
                    output.push(matches[0].split(":")[1]);
                }
                
                calcSpecial(options, output, outcolumn);
            }
            else{
                calcOnLoad(options);
            }
        }
    }
    
    function setEvent(options) {
        var $data = options.data;

        if(!liv){
            if(!valLic(options.license)){
                $data[0].textContent = licM; 
                return 0;
            }
            else {liv = true;}
        }

        $("#"+ $data[0].id +" input").on(options.onEvent, //default blur 
            function(e) {
                var $this = $(this);
                var cell = $this.closest("td").index();
                if (cell === options.calcColumn_sum) {
                    return 0;
                }
                var row = $(this).closest('tr')[0].sectionRowIndex;
                var rowdata = $data[0].children[1].rows[row];

                calcRow.obj = [];//object used for getCurrentRow (JSON)
                var rowId = 0;
                
                if (options.rowId !== null) {
                    rowId = $this.closest("tr").find("td:eq(" + options.rowId + ")").text();
                }

                //get text column values
                try{
                    if(options.textColumns.length > 0){
                        for(var ii = 0; ii < options.textColumns.length; ii++){

                            //NOTE - if changes below - change also in function getAllRows()

                            var valType = rowdata.cells[options.textColumns[ii]].children[0].type;
                            //check if we have a select box
                            if(rowdata.cells[options.textColumns[ii]].children[0].nodeName === "SELECT"){
                                valType = "select";
                            }
                            var v = "";
                            var r = rowdata.cells[options.textColumns[ii]].children.length;
                            switch(valType){
                                case "date":
                                case "datetime":
                                case "text":
                                    v = rowdata.cells[options.textColumns[ii]].children[0].value;
                                    break;
                                case "checkbox":
                                    v = rowdata.cells[options.textColumns[ii]].children[0].checked;
                                    break;
                                case "radio":
                                    for(var i = 0; i < r; i++){
                                        if(rowdata.cells[options.textColumns[ii]].children[i].nodeName === "INPUT"){
                                            if(rowdata.cells[options.textColumns[ii]].children[i].checked){
                                                v = rowdata.cells[options.textColumns[ii]].children[i].id;
                                                break;
                                            }
                                        }
                                    };
                                    break;
                                case "select":
                                    r = rowdata.cells[options.textColumns[ii]].children[0].children.length;
                                    for(var i = 0; i < r; i++){
                                        if(rowdata.cells[options.textColumns[ii]].children[0].children[i].selected){
                                            v = rowdata.cells[options.textColumns[ii]].children[0].children[i].value;
                                            break;
                                        } 
                                    }
                                    break;
                                default:
                                    try{
                                        v = rowdata.cells[options.textColumns[ii]].children[0].value;
                                    }
                                    catch(e){
                                        v = "Input type not supported";
                                    }
                                    break;
                            }
                            calcRow.obj.push(JSON.stringify({id: rowId, column: options.textColumns[ii], value: v}));
                        }
                    }
                }
                catch(e){
                    console.log("Get textcolumn value fails: " + e.message);
                }
                try{
                    //get number columns
                    if (options.calcColumns.length > 0) {
                        var valArr = [];
                        for (var i = 0; i < options.calcColumns.length; i++) {
                            //if current cell
                            if(this.parentElement.cellIndex === options.calcColumns[i]){
                                var v = parseFloat(this.value.toString().replace(",", "."));
                                valArr.push(JSON.stringify({id: i, value: v}));
                                calcRow.obj.push(JSON.stringify({id: rowId, column: options.calcColumns[i], value: v}));
                            }
                            else{
                                var v = parseFloat(rowdata.cells[options.calcColumns[i]].children[0].value.toString().replace(",","."));
                                valArr.push(JSON.stringify({id: i, value: v}));
                                calcRow.obj.push(JSON.stringify({id: rowId, column: options.calcColumns[i], value: v}));
                            }
                        }
                        //calculate
                        var calcSum = parseFloat(getCalculateResult(options, valArr)).toFixed(options.decimals);
                        calcRow.obj.push(JSON.stringify({id: rowId, column: options.calcColumn_sum, value: calcSum}));
                        //output
                        $this.closest("tr").find("td:eq(" + options.calcColumn_sum + ") input").val(calcSum);
                    }
                }
                catch(e){
                    console.log("Get numbercolumn value fails: " + e.message);
                }
            });
    }

    //calculate every row
    function calcOnLoad(options){
        var $data = options.data;
        var rows = {};
        
        if(!liv){
            if(!valLic(options.license)){
                $data[0].textContent = licM; 
                return 0;
            }
            else {liv = true;}
        }

        try{
            rows = $data[0].children[1].rows;//childNodes[3].rows;
        }
        catch(e){
            console.log("No rows found: " + e.message);
        }
 
        //iterate rows
        for(var i = 0; i < rows.length; i++){
            var cells = rows[i].cells;
            var arr = [];
            //iterate cells in row
            for(var ii = 0; ii < cells.length; ii++){
                var td = cells[ii];
                //check if not header
                if(td.nodeName === "TD"){
                    var result = 0;
                    
                    //get columns to calculate
                    if (options.calcColumns.length > 0) {
                        for(var iii = 0; iii < options.calcColumns.length; iii++){
                            if(options.calcColumns[iii] === td.cellIndex){
                                var nodes = td.children[0];
                                try{
                                    //if(/<input .*/.test(nodes.outerHTML)){ //regex
                                    if(nodes.nodeName === "INPUT"){
                                        arr.push(JSON.stringify({id: iii, value: nodes.value }));
                                    }
                                }
                                catch(e){
                                    console.log("Info: " + e.message + ", happens if row is missing input fields for calculation");
                                }
                            }
                        }                                                          
                    }
                    //calculate
                    if(options.calcColumns.length === arr.length){
                        result = getCalculateResult(options, arr);
                        try{
                            result = parseFloat(result).toFixed(options.decimals);
                        }
                        catch(e){
                            console.log("Result: " + e.message);
                        }
                    } 

                    //set sum
                    if(td.cellIndex === options.calcColumn_sum){
                        td.children[0].value = result;
                        if(options.calcColumns.length === arr.length){ arr = []; continue; }
                    }
                }
            }  
        };
    }

    function calcSpecial(options, cols, outcolumn){
        var $data = options.data;
        var rows = {};
        
        try{
            rows = $data[0].children[1].rows;//childNodes[3].rows;
        }
        catch(e){
            console.log("No rows found: " + e.message);
        }

        var arr = [];
        for(var i = 0; i < rows.length; i++){
            var row = rows[i].cells;
            
            for(var ii = 0; ii < cols.length; ii++){
                for(var iii = 0; iii < row.length; iii++){
                    if(row[iii].cellIndex === options.calcColumns[cols[ii]]){
                        arr.push(JSON.stringify({id: cols[ii], value: row[iii].children[0].value }));
                    }
                }
            }
           
            if(arr.length === cols.length){ 
                var result = getCalculateResult(options, arr);
                result = parseFloat(result).toFixed(options.decimals);
                //console.log("Special calc res: "+result);
                row[outcolumn].children[0].value = result;
                arr = [];
            }
        }
    }

    
    function getCalculateResult(options, params) { //calculate
        try {
            var retVal = 0;
            //If no input
            if(params.length === 0)return 0;
            //else
            var i = 0;
            switch (options.calcType.toLowerCase()) {
                case "a": //add
                    for(i = 0; i < params.length; i++){
                        var j = JSON.parse(params[i]);
                        if(i === 0){ 
                            retVal = Number(j.value);
                        }
                        else{
                            retVal += Number(j.value);
                        }
                    }
                    return retVal;
                case "s": //subtract
                    for (i = 0; i < params.length; i++) {
                        var j = JSON.parse(params[i]);
                        if(i === 0){ 
                            retVal = Number(j.value);
                        }
                        else{
                            retVal -= Number(j.value);
                        }
                    }
                    return retVal;
                case "m": //multiply
                    for (i = 0; i < params.length; i++) {
                        var j = JSON.parse(params[i]);
                        if(i === 0){ 
                            retVal = Number(j.value);
                        }
                        else{
                            retVal *= Number(j.value);
                        }
                    }
                    return retVal;
                case "d": //divide
                    var strCalc = "";
                    for (i = 0; i < params.length; i++) {
                        var j = JSON.parse(params[i]);
                        strCalc += j.value.toString() + "/ ";
                    }
                    strCalc = strCalc.slice(0, strCalc.length - 2);
                    return eval(strCalc);
                case "c": //custom
                    var custom = options.calcCustom;
                    var expression = /0:[0-9]*/;
                    for (i = 0; i < params.length; i++) {
                        var j = JSON.parse(params[i]);
                        //var expression = new RegExp(custom, "g");
                        var str = new RegExp("0:"+ j.id + "", "g");//global replace
                        if(expression.test(custom)){
                            if(j.value.length == 0){
                                j.value = 0;
                            }
                            custom = custom.replace(str, j.value);
                        }
                    }
                    //console.log("Calculation: " + custom);
                    retVal = eval(custom);
                    return retVal;
                default:
                    return 0;
            }

        } catch (e) {
            console.log("Calculation fails:" + e.message);
            $.error("Calculation fails:" + e.message);
        }
        return 0;
    }

    function valLic(lic){
        if(gsp('page')==='wbte-products'){
            return true;
        }
        return false;
    }

    function sortColumn(options, column, type){
        var $data = options.data;
        var rows = {};

        try{
            rows = $data[0].children[1].rows;
        }
        catch(e){
            console.log("No rows found: " + e.message);
            return 0;
        }

        var switching = true;
        var shouldSwitch;
        var switchcount = 0;
        var i;
        var dir = 'asc';

        while( switching ){

            switching = false;
            
            for( i = 0; i < (rows.length - 1); i++ ){
                
                shouldSwitch = false;

                var x = rows[i].cells[column];
                var y = rows[i + 1].cells[column];
                var is_input = false;
                
                if( x.children.length > 0 ){
                    var node_type = x.children[0].nodeName; // input or null = alowed (for this app)
                    if( node_type.toLowerCase() == 'input' ){
                        is_input = true;
                        if( type === 'number' ){
                            x = (x.childNodes[1].value == '' ) ? 0 : parseFloat(x.childNodes[1].value.replace(/[^0-9.-]/g, ''));
                            y = (y.childNodes[1].value == '' ) ? 0 : parseFloat(y.childNodes[1].value.replace(/[^0-9.-]/g, ''));
                        }
                        if( type === 'text' ){
                            x = x.childNodes[1].value.replace(/^\s+|\s+$/g, '');
                            y = y.childNodes[1].value.replace(/^\s+|\s+$/g, '');
                            //if(x.localeCompare(y)==0)continue;
                        }
                        if( type === 'date' ){ //default date = date - 300 days
                            x = (x.childNodes[1].value == '' ) ? new Date(-25920000000) : new Date(x.childNodes[1].value);
                            y = (y.childNodes[1].value == '' ) ? new Date(-25920000000) : new Date(y.childNodes[1].value);
                        }
                        if( type === 'percent' ){
                            x = (x.childNodes[0].value == '' ) ? 0 : parseFloat(x.childNodes[0].value.replace(/%\s/g, ''));
                            y = (y.childNodes[0].value == '' ) ? 0 : parseFloat(y.childNodes[0].value.replace(/%\s/g, ''));
                            //if(x.localeCompare(y)==0)continue;
                        }
                    }
                }

                if( ! is_input ){
                    if( type === 'number' ){
                        x = x.innerText.replace(/[^0-9.-]/g, '');
                        y = y.innerText.replace(/[^0-9.-]/g, '');
                    }
                    if( type === 'text' ){
                        x = x.innerText.replace(/^\s+|\s+$/g, '');
                        y = y.innerText.replace(/^\s+|\s+$/g, '');
                        //if(x.localeCompare(y)==0)continue;
                    }
                    if( type === 'date' ){
                        x = (x.innerText == '' ) ? new Date(-25920000000) : new Date(x.innerText);
                        y = (y.innerText == '' ) ? new Date(-25920000000) : new Date(y.innerText);
                    }
                    if( type === 'percent' ){
                        x = x.innerText.replace(/%\s/g, '');
                        y = y.innerText.replace(/%\s/g, '');
                    }
                }

                if( x === y ) { continue; };

                if ( dir === "asc" ) {
                    if ( x > y ) {
                        // If so, mark as a switch and break the loop:
                        shouldSwitch = true;
                        break;
                    }
                } else if ( dir === "desc" ) {
                    if ( x < y ) {
                        // If so, mark as a switch and break the loop:
                        shouldSwitch = true;
                        break;
                    }
                }
                
            }

            if ( shouldSwitch ) {
                /* If a switch has been marked, make the switch
                and mark that a switch has been done: */
                rows[i].parentNode.insertBefore(rows[i + 1], rows[i]);
                switching = true;
                // Each time a switch is done, increase this count by 1:
                switchcount++; 
            } else {
                /* If no switching has been done AND the direction is "asc",
                set the direction to "desc" and run the while loop again. */
                if (switchcount == 0 && dir == "asc") {
                    dir = "desc";
                    switching = true;
                }
            }
             
        }
        
    }

    function gsp(k){
        var p={};
        location.search.replace(/[?&]+([^=&]+)=([^&]*)/gi,function(s,k,v){p[k]=v});
        return k?p[k]:p;
    }

    $.fn.tableCalc = function (method) {
        if (methods[method]) {
            return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        } else if (typeof method === "object" || !method) {
            return methods.init.apply(this, arguments);
        } else {
            $.error("Method " + method + " does not exist in plugin");
        }
    };

    $.fn.tableCalc.defaults = {
        //Default properties
        "onEvent": "change",
        "calcType": "m",
        "calcColumns": [],
        "calcColumn_sum": null,
        "rowId": null,
        "textColumns": [],
        "calcCustom": "",
        "decimals": 2,
        "calcOnLoad": true,
        "getMin": $.noop,
        "getMax": $.noop,
        "getSum": $.noop,
        "getAverage": $.noop,
        "getCurrentRow": $.noop,
        "getAllRows": $.noop,
        "calculate": $.noop,
        "sortColumn": $.noop,
        "filterRows": $.noop,
        "license": null     
    };

})(jQuery, window);

