<?php
/**
 * Bulk Table Editor custom 
 *
 * @package BulkTableEditor/includes
 */

defined( 'ABSPATH' ) || exit;

require_once __DIR__ . '/class-wbte-functions.php';

const CAS_WBTE_CUSTOM_FIELDS = 'Wbte_Custom_Fields';

/**
 * Custom field class
 */
class Wbte_Custom_Fields {

	/**
	 * Get all custom fields from settings
	 */
	public static function wbte_get_custom_fields_array() {

		$options       = get_option('wbte_options');
		$custom_fields = array();

		for ( $i = 1; $i <= 5; $i++ ) {
			
			$custom_field = new stdClass();
			$custom_field->name = isset( $options["wbte_custom_field_{$i}_name"] ) ? $options["wbte_custom_field_{$i}_name"] : '';
			$custom_field->type = isset( $options["wbte_custom_field_{$i}_type"] ) ? $options["wbte_custom_field_{$i}_type"] : '';
			$custom_field->slug = isset( $options["wbte_custom_field_{$i}_slug"] ) ? $options["wbte_custom_field_{$i}_slug"] : '';
			$custom_field->use  = isset( $options["wbte_use_custom_field_{$i}"] ) ? $options["wbte_use_custom_field_{$i}"] : '';

			if ( 'yes' === $custom_field->use ) {
				$custom_fields[] = $custom_field;
			}
		}

		return $custom_fields;
	}

	/**
	 * Get table and its data
	 */
	public static function wbte_get_custom_table() {

		$functions   = new WbteFunctions();
		$page_num    = filter_input( 1, 'paged', FILTER_SANITIZE_NUMBER_INT ) ? filter_input( 1, 'paged', FILTER_SANITIZE_NUMBER_INT ) : 1;
		$view        = filter_input( 1, 'view', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$product_obj = $functions->wbte_get_woo_products();
		$products    = $product_obj['products'];
		$total_pages = $product_obj['max_num_pages'];
		
		?>
		<div class="display-desktop"> <!-- Table -->
			<div class="cas-action-box" style="width:99%;margin-right:20px;">
				<div style="text-align:right;padding-right:10px;">
					<button class="button" id="reset-values-footer" onclick="wbte_reset_table_values();" style="width:150px;"><i class="fas fa-undo"></i> <?php esc_attr_e( 'Undo', 'woo-bulk-table-editor' ); ?></button>
					<button class="button button-primary" onclick="wbteSaveCustomTable();" style="width:150px;"><i class="fas fa-database"></i> <?php esc_attr_e( 'Save Changes', 'woo-bulk-table-editor' ); ?></button>
				</div>
			</div>
			<table class="wp-list-table widefat fixed striped posts" id="wbtetable_custom" style="width:100%;">
				<thead>
					<?php self::wbte_get_table_head( $functions ); ?>
				</thead>
				<tbody id="the-list"> 
					<?php self::wbte_loop_products( $products, $functions ); ?>
				</tbody>
				<tfoot><?php wp_nonce_field( 'wbte_id' ); ?></tfoot>
			</table>
			
			<div id="wbte-paging-bottom2" class="cas-div-paging" style="margin-top: 6px;">
				<?php $functions->wbte_get_pagination( $page_num, $total_pages ); ?>
			</div>

			</div>
		<div>
		<!-- FINAL END DIV -->
		</div>		
		<?php
	}

	/**
	 * Loop products
	 */
	private static function wbte_loop_products( $products, $functions ) {

		$subscriptions_exists = class_exists( 'WC_Product_Subscription' );
	
		foreach ( $products as $product ) {
			if ( $product instanceof WC_Product ) {
				// Determine if the product is a variation or simple product
				$is_variation = $product instanceof WC_Product_Variation;
		
				// Process subscriptions or simple products
				if ( !$is_variation && ( $product->is_type( 'simple' ) || ( $subscriptions_exists && $product->is_type( 'subscription' ) ) ) ) {
					self::wbte_get_table_row( $product, false, $functions );
				// Process variations (including subscription variations)
				} elseif ( $is_variation || ( $subscriptions_exists && $product->is_type( 'variable-subscription' ) ) ) { 
					self::wbte_get_table_row( $product, true, $functions );
				} else {
					// Other cases e.g. grouped products
					self::wbte_get_table_row( $product, false, $functions );
				}
			} 
		}
		
	}

	/**
	 * Get table head
	 */
	public static function wbte_get_table_head( $functions ) {

		$custom_fields = self::wbte_get_custom_fields_array();
		$sort_count    = 4;

		?>
		<tr>
			<th class="th-first" style="width:20px;"><input type="checkbox" id="checkall" onclick="checkVisible();"></th>
			<th>
				<a href="#" onclick="sort(1,'text');"><i id="s1" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Name', 'woo-bulk-table-editor' ); ?></a>
			</th>
			<th style="width:20px;"></th>
			<th style="width:100px;">
				<a href="#" onclick="sort(3,'text');"><i id="s3" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Type', 'woo-bulk-table-editor' ); ?></a></th>
			<th style="width:300px;"><?php esc_html_e( 'Attributes', 'woo-bulk-table-editor' ); ?></a></th>
			<?php foreach ( $custom_fields as $custom ) : ?>
				<?php $sort_count++; ?>
				<th><a href="#" onclick="sort(<?php echo esc_attr( $sort_count ); ?>, '<?php echo esc_attr( $custom->type ); ?>');"><i id="s<?php echo esc_attr( $sort_count ); ?>" class="fas fa-sort cas-i-space"></i><?php echo esc_attr( $custom->name ); ?></a></th>
			<?php endforeach; ?>
		</tr>
		
		<tr class="th-info">
			<th></th>
			<th style="vertical-align: middle;">
				<strong><?php esc_html_e( 'Bulk Editor', 'woo-bulk-table-editor' ); ?></strong><br />
				<span class="th-desc"><?php esc_html_e( 'Enter values (input) then select bulk function.', 'woo-bulk-table-editor' ); ?>
				</span>
			</th>
			<th></th>
			<th></th>
			<th>
				<?php echo wp_kses( $functions->wbte_get_attributes_dropdown_select( 'bulk_add_attribute' ), $functions->wbte_get_allowed_html() ); ?>
				<select class="input-select" id="wbte_attributes_action" onchange="wbte_bulk_add_attributes();" title="<?php esc_html_e( 'Bulk Add Attributes (Not for Variations)', 'woo-bulk-table-editor' ); ?>">
					<option value=""><?php esc_html_e( 'Select Action', 'woo-bulk-table-editor' ); ?></option>
					<option value="add"><?php esc_html_e( 'Add', 'woo-bulk-table-editor' ); ?></option>
					<option value="set"><?php esc_html_e( 'Set', 'woo-bulk-table-editor' ); ?></option>
					<option value="remove"><?php esc_html_e( 'Remove', 'woo-bulk-table-editor' ); ?></option>
					<option value="clear"><?php esc_html_e( 'Clear', 'woo-bulk-table-editor' ); ?></option>
				</select>
			</th>
			<?php foreach ( $custom_fields as $custom ) : ?>
				<th>
					<input type="text" name="<?php echo esc_attr( $custom->slug ); ?>_value" id="<?php echo esc_attr( $custom->slug ); ?>_value" class="input-txt"><br/>
					<select name="<?php echo esc_attr( $custom->slug ); ?>_select" id="<?php echo esc_attr( $custom->slug ); ?>_select" class="input-select" onchange="wbteBulkCustomValues('<?php echo esc_js( $custom->slug ); ?>');">
						<?php $functions->wbte_get_bulk_options( 'customs' ); ?>
					</select>
				</th>
			<?php endforeach; ?>
		</tr>
		<?php
	}

	/**
	 * Get table rows
	 */
	public static function wbte_get_table_row( $product, $is_variable, $functions ) {

		$product_name = $product->get_name();
		$attributes   = $product->get_attributes();
		$product_id   = $product->get_id();
		$type         = ( $is_variable ) ? 'var' : 'prod';
		$tag_ids      = $product->get_tag_ids();

		if ( $is_variable ) {
			$product_id = $product->get_parent_id();
			$tag_ids    = wc_get_product( $product_id )->get_tag_ids();

			if ( ! empty( $attributes ) && is_array( $attributes ) ) {
				foreach ( $attributes as $key => $val ) {
					if ( ! empty( $val ) ) {
						$product_name .= ', ' . ucfirst( $val );
					}
				}
			}
		}

		$tags          = $functions->wbte_get_product_tags( $tag_ids );
		$custom_fields = self::wbte_get_custom_fields_array();
	
		?>
		
		<tr class="lozad" style="vertical-align: top;" data-type="<?php echo esc_attr( $type ); ?>">
			<td id="<?php echo esc_attr( $product->get_id() ); ?>" class="cas-td-chk">
				<form id="frm_<?php echo esc_attr( $product->get_id() ); ?>_<?php echo esc_attr( wp_rand( 1, 50000 ) ); ?>">
				<input name="id" value="<?php echo esc_attr( $product->get_id() ); ?>" type="checkbox">
				<input type="hidden" name="saleprice" value="<?php echo esc_attr( $product->get_sale_price() ); ?>">
				<input type="hidden" name="tags" value="<?php echo esc_attr( $tags ); ?>">
				<input type="hidden" name="stock_status" value="<?php echo esc_attr( $product->get_stock_status() ); ?>">
				<input type="hidden" name="brands" value="<?php echo esc_attr( $functions->wbte_get_product_brands( $product_id, 'slug' ) ); ?>">
				<input type="hidden" name="parent_id" value="<?php echo esc_attr( $is_variable ? $product->get_parent_id() : '' ); ?>">
				<?php echo wp_kses( (string) self::wbte_get_product_attributes_hidden_input( $product ), 'post' ); ?>
			</td>
			<td>
				<?php
				if ( ! $is_variable ) {
					?>
					<input type="text" class="input-txt-name" name="product_name" value="<?php echo esc_html( $product_name ); ?>">
					<input type="hidden" name="name_update" value="true">
					<?php
				} else {
					?>
					<input type="hidden" name="product_name" value="<?php echo esc_html( $product_name ); ?>">
					<input type="hidden" name="name_update" value="false">
					<?php echo esc_attr( $product_name ); ?>
					<?php
				}
				?>
			</td>
			<td>
				<a href="<?php echo esc_url( get_admin_url() . 'post.php?post=' . $product_id . '&action=edit' ); ?>" target="_new" title="<?php esc_attr_e( 'Edit product', 'woo-bulk-table-editor' ); ?>">
					<i class="fas fa-edit <?php echo esc_attr( ( $product->is_on_sale() ) ? 'cas-fa-sale' : '' ); ?>"></i>
				</a>
			</td>
			<td><?php echo esc_attr( ucfirst( $product->get_type() ) ); ?></td>
			<td>
				<?php self::wbte_get_product_attributes( $product ); ?>
			</td>
			<?php foreach ( $custom_fields as $custom ) : ?>
				<td>
					<input type="text" class="input-txt-name" data-iscustom="1" name="<?php echo esc_attr( $custom->slug ); ?>" value="<?php echo esc_attr( get_post_meta( $product->get_id(), $custom->slug, true ) ); ?>">
				</td>
			<?php endforeach; ?>
			</form>
		</tr>
		<?php
	}

	/**
	 * Check if dark mode is enabled
	 */
	public static function wbte_is_dark_mode() {
		$options = get_option('wbte_options');
		return isset( $options['wbte_use_dark_mode'] ) && 'yes' === $options['wbte_use_dark_mode'];
	}

	/**
	 * Get product attributes (list per product).
	 */
	public static function wbte_get_product_attributes( $product ) {
		if ( empty( $product ) ) {
			return '';
		}

		$attributes      = $product->get_attributes();
		$attribute_list  = '';
		$attr_input_list = '';
		$is_variation    = $product->is_type('variation');
		$variation_css   = self::wbte_is_dark_mode() ? 'background:#333;color:white' : 'background:white;color:black';

		if ( $is_variation ) {
			$variation_attributes = $product->get_variation_attributes();
			$variation_css        = self::wbte_is_dark_mode() ? 'background:#333;color:silver' :'background:whitesmoke;color:gray';
			$output               = '';

			foreach ( $variation_attributes as $attribute_slug => $attribute_value ) {
				$taxonomy = str_replace( 'attribute_', '', $attribute_slug );

				// Check if the attribute is a taxonomy-based attribute
				if ( taxonomy_exists( $taxonomy ) ) {
					$term = get_term_by( 'slug', $attribute_value, $taxonomy );
					if ( $term ) {
						$output .= '<strong>' . wc_attribute_label( $taxonomy ) . '</strong>: ' . esc_html( $term->name ) . '<br>';
					}
				} else {
					// Custom attribute
					$output .= '<strong>' . wc_attribute_label( $taxonomy ) . '</strong>: ' . esc_html( $attribute_value ) . '<br>';
				}
			}
			$attribute_list = $output;
		}

		if ( ! empty( $attributes ) && ! $is_variation ) {
			foreach ( $attributes as $attribute ) {
				if ( $attribute->is_taxonomy() ) {
					// Taxonomy-based attribute
					$terms  = wp_get_post_terms( $product->get_id(), $attribute->get_name(), 'all' );
					$values = array();
					foreach ( $terms as $term ) {
						$values[] = esc_html( $term->name );
					}
					if ( ! empty( $values ) ) {
						$attribute_list  .= '<strong>' . wc_attribute_label( $attribute->get_name() ) . '</strong>: ' . implode( ', ', $values ) . '<br>';
						$attr_input_list .= wc_attribute_label( $attribute->get_name() ) . '=' . implode( ',', $values ) . ';';
					}
				} else {
					// Custom attribute
					$values = $attribute->get_options();
					if ( ! empty( $values ) ) {
						$attribute_list  .= '<strong>' . wc_attribute_label( $attribute->get_name() ) . '</strong>: ' . implode( ', ', $values ) . '<br>';
						$attr_input_list .= wc_attribute_label( $attribute->get_name() ) . '=' . implode( ',', $values ) . ';';
					}
				}
			}
		}

		?>
		<input type="hidden" name="attributes" value="<?php echo esc_attr( strtolower( $attr_input_list ) ); ?>" data-is-variation="<?php echo esc_attr( $is_variation ); ?>">
		<div id="<?php echo esc_attr( $product->get_id() ); ?>_attributes_txt" style="padding-left:4px;<?php echo esc_attr( $variation_css ); ?>; overflow-y:auto; max-height:150px;">
			<?php 
				echo wp_kses( $attribute_list, array(
					'strong' => array(),
					'br'     => array(),
				) ); 
			?>
		</div>
		<?php
	}

	/**
	 * Get all unique product attributes with transients, ensuring data type consistency.
	 */
	public static function wbte_get_all_product_attributes() {
		
		$transient_key        = 'wbte_all_product_attributes';
		$transient_expiration = 20 * MINUTE_IN_SECONDS;
		$product_attributes   = get_transient( $transient_key );

		if ( false === $product_attributes ) {
			$product_attributes = array();
			$all_taxonomies     = wc_get_attribute_taxonomy_names();

			if ( is_wp_error( $all_taxonomies ) ) {
				error_log( 'Error fetching attribute taxonomies: ' . $all_taxonomies->get_error_message() );
				return $product_attributes;
			}

			$query = new WC_Product_Query( array(
				'limit'  => -1,
				'return' => 'ids',
				'status' => 'publish',
				'meta_query' => array(
					array(
						'key'     => '_product_attributes',
						'compare' => 'EXISTS',
					),
				),
			));
			
			$product_ids = $query->get_products();

			if ( ! empty( $product_ids ) ) {
				foreach ( $product_ids as $product_id ) {
					$product = wc_get_product( $product_id );
					
					if ( $product && is_a( $product, 'WC_Product' ) ) {
						$attributes = $product->get_attributes();

						foreach ( $attributes as $attribute ) {
							$name  = $attribute->get_name();
							$label = wc_attribute_label( $name );
							$slug  = sanitize_title( $label ); 

							// Initialize array for the attribute if it doesn't exist
							if ( ! isset( $product_attributes[ $slug ] ) ) {
								$product_attributes[ $slug ] = array(
									'label'   => $label,
									'options' => array()
								);
							}

							if ( $attribute->is_taxonomy() ) {
								$terms = get_terms( array(
									'taxonomy'   => $name,
									'hide_empty' => false,
									'fields'     => 'names',
									'number'     => 100,
								));
								if ( ! is_wp_error( $terms ) && is_array( $terms ) ) {
									foreach ( $terms as $term ) {
										if ( is_string( $term ) ) { // Ensure the term is a string
											$product_attributes[ $slug ]['options'][] = $term;
										}
									}
								}
							} else {
								$options = $attribute->get_options();
								if ( is_array( $options ) ) {
									foreach ( $options as $option ) {
										if ( is_string( $option ) ) { // Ensure each option is a string
											$product_attributes[ $slug ]['options'][] = $option;
										}
									}
								}
							}
						}
					}
				}

				foreach ( $all_taxonomies as $taxonomy ) {
					$label = wc_attribute_label( $taxonomy );
					$slug  = sanitize_title( $label ); 

					if ( ! isset( $product_attributes[ $slug ] ) ) {
						$product_attributes[ $slug ] = array(
							'label'   => $label,
							'options' => array()
						);
					}

					$terms = get_terms( array(
						'taxonomy'   => $taxonomy,
						'hide_empty' => false,
						'fields'     => 'names',
						'number'     => 100,
					));
					if ( ! is_wp_error( $terms ) && is_array( $terms ) ) {
						foreach ( $terms as $term ) {
							if ( is_string( $term ) ) { // Ensure the term is a string
								$product_attributes[ $slug ]['options'][] = $term;
							}
						}
					}
				}

				foreach ( $product_attributes as &$data ) {
					if ( is_array( $data['options'] ) ) {
						$data['options'] = array_unique( $data['options'] );
						sort( $data['options'] );
					} else {
						error_log( 'Notice: Non-array found in options for attribute ' . $data['label'] );
						$data['options'] = array();
					}
				}
			}

			set_transient( $transient_key, $product_attributes, $transient_expiration );
		}

		return $product_attributes;
	}

	/**
	 * Get product attributes (input hidden) - to use in filters.
	 */
	public static function wbte_get_product_attributes_hidden_input( $product ) {
		
		if ( empty( $product ) ) {
			return '';
		}

		$attributes      = $product->get_attributes();
		$attr_input_list = '';
		$is_variation    = $product->is_type('variation');

		if ( ! empty( $attributes ) && ! $is_variation ) { 
			foreach ( $attributes as $attribute ) {
				if ( $attribute->is_taxonomy() ) {
					// Taxonomy-based attribute
					$terms = wp_get_post_terms( $product->get_id(), $attribute->get_name(), array( 'fields' => 'names' ) );
					if ( ! is_wp_error( $terms ) && ! empty( $terms ) ) {
						$values           = array_map( 'esc_html', $terms ); // Sanitize each term
						$attr_input_list .= wc_attribute_label( $attribute->get_name() ) . '=' . implode( ',', $values ) . ';'; //Use wc_attribute_label
					}
				} else {
					// Custom attribute
					$values = $attribute->get_options();
					if ( is_array( $values ) && ! empty( $values ) ) {
						$values           = array_map( 'esc_html', $values ); // Sanitize each value
						$attr_input_list .= $attribute->get_name() . '=' . implode( ',', $values ) . ';'; //Use only name for filter to work
					}
				}
			}
		} else if ( ! empty( $attributes ) && $is_variation ) {
			foreach ( $attributes as $key => $val ) {
				if ( ! empty( $val ) && is_string( $val ) ) {
					// Remove the 'pa_' prefix and sanitize the key and value
					$clean_key        = str_replace( 'pa_', '', $key );
					$attr_input_list .= esc_attr( $clean_key ) . '=' . esc_html( $val ) . ';';
				}
			}
		}

		// Render the hidden input field with the formatted attribute list
		?>
		<input type="hidden" name="p_attributes" value="<?php echo esc_attr( strtolower( $attr_input_list ) ); ?>">
		<?php
	}

}

