<?php
/**
 * Bulk Table Editor extended templates
 *
 * @package BulkTableEditor/includes
 */

defined( 'ABSPATH' ) || exit;

require_once __DIR__ . '/class-wbte-functions.php';

/**
 * Class for extended templates
 */
class WbteExtended {

	/**
	 * WbteFunctions
	 *
	 * @var var $functions.
	 */
	public $functions;

	/**
	 * WbteTemplates
	 *
	 * @var var $templates.
	 */
	public $templates;

	/**
	 * WbteOptions
	 *
	 * @var var $wbte_options.
	 */
	public $wbte_options;


	/**
	 * Constructor
	 */
	public function __construct() {
		$this->functions    = new WbteFunctions();
		$this->wbte_options = get_option( 'wbte_options' );
	}

	/**
	 * Get the extended table
	 *
	 */
	public function wbte_get_table( $products, $view, $page_num, $max_pages ) {
		
		?>
		<div class="display-desktop"> <!-- Table -->
			<table class="wp-list-table widefat fixed striped posts" id="wbtetable_ext" style="width:100%;">
				<thead>
					<?php $this->wbte_get_table_head(); ?>
					<?php $this->wbte_get_table_head_info(); ?>
				</thead>
				<tbody id="the-list"> 
					<?php $this->wbte_loop_products( $products ); ?>
				</tbody>
					<?php $this->wbte_get_table_footer(); ?>
			</table>
			
			<div id="wbte-paging-bottom2" class="cas-div-paging" style="margin-top: 6px;">
				<?php $this->functions->wbte_get_pagination( $page_num, $max_pages ); ?>
			</div>

			</div>
		<div>
		<?php

	}

	/**
	 * Loop products
	 */
	private function wbte_loop_products( $products ) {

		$subscriptions_exists = class_exists( 'WC_Product_Subscription' );
	
		foreach ( $products as $product ) {
			if ( $product instanceof WC_Product ) {
				// Determine if the product is a variation or simple product
				$is_variation = $product instanceof WC_Product_Variation;
		
				// Process subscriptions or simple products
				if ( !$is_variation && ( $product->is_type( 'simple' ) || ( $subscriptions_exists && $product->is_type( 'subscription' ) ) ) ) {
					$this->wbte_get_table_row( $product, false );
				// Process variations (including subscription variations)
				} elseif ( $is_variation || ( $subscriptions_exists && $product->is_type( 'variable-subscription' ) ) ) {
					$this->wbte_get_table_row( $product, true );
				} else {
					// Other cases e.g. grouped products
					$this->wbte_get_table_row( $product, false );
				}
			} 
		}
		
	}

	/**
	 * Get table head
	 */
	public function wbte_get_table_head() {

		$wbte_options = get_option( 'wbte_options' );
		$disable_desc = ( strlen( $wbte_options[ 'wbte_disable_description' ] ) > 0 ) ? $wbte_options[ 'wbte_disable_description' ] : 'no';
		$desc_label   = ( 'no' === $disable_desc ) ? __( 'Description ID SKU', 'woo-bulk-table-editor' ) : __( 'Show ID & SKU', 'woo-bulk-table-editor' );
		
		?>
		<tr>
			<th class="th-first"><input type="checkbox" id="checkall" onclick="checkVisible();"></th>
			<th style="width:20%;">
				<a href="#" onclick="sort(1,'text');"><i id="s1" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Name', 'woo-bulk-table-editor' ); ?></a>
				<input type="checkbox" id="wbte-chk-show-desc" onclick="wbte_show_hide_desc();"> <?php echo esc_attr( $desc_label ); ?>
			</th>
			<th class="th-center" style="width: 90px;"><i class="fas fa-star" data-tip="Featured"></i></th>
			<th class="th-center" style="width:12%;"><a href="#" onclick="sort(3,'text');"><i id="s3" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'SKU', 'woo-bulk-table-editor' ); ?></a></th>
			<th class="th-center" style="width:12%;"><a href="#" onclick="sort(4,'text');"><i id="s4" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Tags', 'woo-bulk-table-editor' ); ?></a></th>
			<th class="th-center"><?php esc_html_e( 'Backorders', 'woo-bulk-table-editor' ); ?></th>
			<th class="th-center"><?php esc_html_e( 'Stock Status', 'woo-bulk-table-editor' ); ?></th>
			<th class="th-center"><?php esc_html_e( 'Visibility', 'woo-bulk-table-editor' ); ?></th>
			<th class="th-center"><a href="#" onclick="sort(8,'number');"><i id="s7" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Weight', 'woo-bulk-table-editor' ); ?></a></th>
			<th class="th-center"><a href="#" onclick="sort(9,'number');"><i id="s8" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Length', 'woo-bulk-table-editor' ); ?></a></th>
			<th class="th-center"><a href="#" onclick="sort(10,'number');"><i id="s9" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Width', 'woo-bulk-table-editor' ); ?></a></th>
			<th class="th-center"><a href="#" onclick="sort(11,'number');"><i id="s10" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Height', 'woo-bulk-table-editor' ); ?></a></th>
			<th class="th-center"><?php esc_html_e( 'Shipping', 'woo-bulk-table-editor' ); ?></th>
			<th class="th-center" style="width:100px;"><i class="fas fa-image"></i></th>
	
		</tr>

		<?php
	}

	/**
	 * Get table head info
	 */
	public function wbte_get_table_head_info() {

		$auto_date  = 'date';
		$auto_focus = '';

		?>
			<tr class="th-info">
				<th></th>
				<th style="vertical-align: middle;"><strong><?php esc_html_e( 'Bulk Editor', 'woo-bulk-table-editor' ); ?></strong><br /><span class="th-desc"><?php esc_html_e( 'Fill in values to apply to all visible products. Double click name for description, ID and SKU.', 'woo-bulk-table-editor' ); ?></span></th>
				<th style="vertical-align: bottom;">
					<select name="featured_select_type" id="featured_select_type" class="input-select" onchange="bulkSetValues('featured_select', 'ext');">
						<?php $this->functions->wbte_get_bulk_options( 'featured' ); ?>
					</select>
				</th>
				<th style="vertical-align: top;"><input type="text" name="sku_select" id="sku_select" class="input-txt" onmouseover="<?php echo esc_js( $this->functions->wbte_get_autofocus_value( $auto_focus ) ); ?>">
				<br/>
					<select name="sku_select_type" id="sku_select_type" class="input-select" onchange="bulkSetValues('sku_select', 'ext');">
						<?php $this->functions->wbte_get_bulk_options( 'sku' ); ?>
					</select>
				</th>
				<th style="vertical-align: top;"><input type="text" name="tags_select" id="tags_select" class="input-txt" onmouseover="<?php echo esc_js( $this->functions->wbte_get_autofocus_value( $auto_focus ) ); ?>">
				<br/>
					<select name="tags_select_type" id="tags_select_type" class="input-select" onchange="bulkSetValues('tags_select', 'ext');">
						<?php $this->functions->wbte_get_bulk_options( 'tags' ); ?>
					</select>
				</th>
				<th style="vertical-align: bottom;">
					<select name="backorder_select_type" id="backorder_select_type" class="input-select" onchange="bulkSetValues('backorder_select', 'ext');">
						<?php $this->functions->wbte_get_bulk_options( 'backorder' ); ?>
					</select>
				</th>
				
				<th style="vertical-align: bottom;">
					<select name="instock_select_type" id="instock_select_type" class="input-select" onchange="bulkSetValues('instock_select', 'ext');">
						<?php $this->functions->wbte_get_bulk_options( 'instock' ); ?>
					</select>
				</th>
				<th style="vertical-align: bottom;">
					<select name="visibility_select_type" id="visibility_select_type" class="input-select" onchange="bulkSetValues('visibility_select', 'ext');">
						<?php $this->functions->wbte_get_bulk_options( 'visibility' ); ?>
					</select>
				</th>
				<th style="vertical-align: top;"><input type="number" name="weight_select" id="weight_select" class="input-txt" onmouseover="<?php echo esc_js( $this->functions->wbte_get_autofocus_value( $auto_focus ) ); ?>">
				<br/>
					<select name="weight_select_type" id="weight_select_type" class="input-select" onchange="bulkSetValues('weight_select', 'ext');">
						<?php $this->functions->wbte_get_bulk_options( 'numbers' ); ?>
					</select>
				</th>
				<th style="vertical-align: top;"><input type="number" name="length_select" id="length_select" class="input-txt" onmouseover="<?php echo esc_js( $this->functions->wbte_get_autofocus_value( $auto_focus ) ); ?>">
				<br/>
					<select name="length_select_type" id="length_select_type" class="input-select" onchange="bulkSetValues('length_select', 'ext');">
						<?php $this->functions->wbte_get_bulk_options( 'numbers' ); ?>
					</select>
				</th>
				<th style="vertical-align: top;"><input type="number" name="width_select" id="width_select" class="input-txt" onmouseover="<?php echo esc_js( $this->functions->wbte_get_autofocus_value( $auto_focus ) ); ?>">
				<br/>
					<select name="width_select_type" id="width_select_type" class="input-select" onchange="bulkSetValues('width_select', 'ext');">
						<?php $this->functions->wbte_get_bulk_options( 'numbers' ); ?>
					</select>
				</th>
				<th style="vertical-align: top;"><input type="number" name="height_select" id="height_select" class="input-txt" onmouseover="<?php echo esc_js( $this->functions->wbte_get_autofocus_value( $auto_focus ) ); ?>">
				<br/>
					<select name="height_select_type" id="height_select_type" class="input-select" onchange="bulkSetValues('height_select', 'ext');">
						<?php $this->functions->wbte_get_bulk_options( 'numbers' ); ?>
					</select>
				</th>

				<th style="vertical-align: bottom;">
					<select name="shipping_select_type" id="shipping_select_type" class="input-select" onchange="bulkSetValues('shipping_select', 'ext');">
						<?php $this->functions->wbte_get_bulk_options( 'shipping' ); ?>
					</select>
				</th>

				<th style="vertical-align: top;">
					<button type="button" id="saveallExt" class="button button-primary" onclick="saveAllExt(this);" style="width:100%;"><i class="fas fa-database"></i> <?php esc_html_e( 'Save', 'woo-bulk-table-editor' ); ?></button>
					<div class="wbte-saving" id="wbte-saving">
						<progress id="pbar-saving" class="pbar-saving" value="0" max="100"></progress>
					</div>
					<div class="wbte-bulk-images" id="wbte-bulk-images">
						<a href="#" onclick="wbteBulkSetImages();"><i class="fas fa-edit" style="padding-right:5px;padding-left:9px;font-size:medium;"></i></a>
						<a href="#" onclick="wbteBulkRemoveImages();"> <i class="fas fa-trash" style="padding-right:1px;font-size:medium;padding-top:9px;"></i> </a>
					</div>
				</th>
			</tr>

		<?php
	}

	/**
	 * Get table row
	 *
	 * @param var $product object.
	 * @param var $has_attributes bool.
	 */
	public function wbte_get_table_row( $product, $has_attributes ) {

		$product_name = $product->get_title();
		$product_id   = $product->get_id();
		$disable_desc = ( strlen( $this->wbte_options[ 'wbte_disable_description' ] ) > 0 ) ? $this->wbte_options[ 'wbte_disable_description' ] : 'no';
		$type         = ( $has_attributes ) ? 'var' : 'prod';
		$tag_ids      = $product->get_tag_ids();

		if ( $has_attributes ) {

			$attributes = $product->get_attributes();

			if ( isset( $attributes ) && is_array( $attributes ) ) {
				foreach ( $attributes as $key => $val ) {
					if ( isset( $val ) && is_string( $val ) && strlen( $val ) > 0 ) {
						$product_name .= ', ' . $val;
					}
				}
				$product_name = ucwords( $product_name );
			}

			$product_id = $product->get_parent_id();
			
			if ( 0 === $product_id ) {
				$product_id = $product->get_id();
			} else {
				$tag_ids = wc_get_product( $product_id )->get_tag_ids();
			}
		
		}

		$auto_focus = '';
		$tags       = $this->functions->wbte_get_product_tags( $tag_ids );
		$on_sale    = $product->is_on_sale();
		?>
		<tr class="lozad" style="vertical-align: top;" data-type="<?php echo esc_attr( $type ); ?>">
			<td id="<?php echo esc_attr( $product->get_id() ); ?>" style="padding-top:10px;">
				<form id="frm_<?php echo esc_attr( $product->get_id() ); ?>_<?php echo esc_attr( wp_rand( 1, 50000 ) ); ?>" method="post">
				<input name="id" value="<?php echo esc_attr( $product->get_id() ); ?>" type="checkbox">
				<input type="hidden" name="saleprice" value="<?php echo esc_attr( $product->get_sale_price() ); ?>">
				<input type="hidden" name="parent_id" value="<?php echo esc_attr( $has_attributes ? $product->get_parent_id() : '' ); ?>">
				<input type="hidden" name="stock_status" value="<?php echo esc_attr( $product->get_stock_status() ); ?>">
				<input type="hidden" name="brands" value="<?php echo esc_attr( $this->functions->wbte_get_product_brands( $product->get_id() ) ); ?>">
				<?php echo wp_kses( Wbte_Custom_Fields::wbte_get_product_attributes_hidden_input( $product ), 'post' ); ?>
			</td>
			<td style="display:flex;width:98%" class="<?php echo esc_attr( ( $product->get_sale_price() > 0 ) ? 'cas-td-sale' : '' ); ?>" ondblclick="wbte_show_desc('<?php echo esc_attr( $product->get_id() ); ?>');">
				<?php
				if ( ! $has_attributes ) {
					?>
					<div style="flex:1;width:100%;">
					<input type="text" class="input-txt-name" name="product_name" value="<?php echo esc_html( $product_name ); ?>">
					<input type="hidden" name="name_update" value="true">
					<?php
				} else {
					?>
					<div style="flex:1;width:100%;padding-top:5px;">
					<input type="hidden" name="product_name" value="<?php echo esc_html( $product_name ); ?>">
					<input type="hidden" name="name_update" value="false">
					<?php echo esc_html( $product_name ); ?>
					
					<?php
				}
				?>
					<?php if ( 'no' === $disable_desc ) : ?>
					<div class="wbte-prod-desc-hide" id="wbte_desc_<?php echo esc_attr( $product->get_id() ); ?>" style="padding-top:4px;">
						<textarea id="desc_<?php echo esc_attr( $product->get_id() ); ?>" class="wbte-tiny-editor"><?php echo wp_kses( $product->get_description(), 'post' ); ?></textarea>
					</div>
					<?php endif; ?>
					<ul class="wbte-ul-info-hide" id="wbte_ul_desc_<?php echo esc_attr( $product->get_id() ); ?>">
						<li><?php esc_html_e( 'ID:', 'woo-bulk-table-editor' ); ?> <?php echo esc_attr( $product->get_id() ); ?></li>
						<li><?php esc_html_e( 'SKU:', 'woo-bulk-table-editor' ); ?> <?php echo esc_attr( $product->get_sku() ); ?></li>
					</ul>
				</div>
				<div style="flex: 0;text-align:center;">
					<a href="<?php echo esc_url( get_admin_url() . 'post.php?post=' . $product_id . '&action=edit' ); ?>" target="_new" title="<?php esc_attr_e( 'Edit product', 'woo-bulk-table-editor' ); ?>">
						<i class="fas fa-edit <?php echo esc_attr( ( $on_sale ) ? 'cas-fa-sale' : '' ); ?>"></i>
					</a>
					<button name="btn_desc" id="btn_desc_<?php echo esc_attr( $product->get_id() ); ?>" class="btn-desc-hide" onclick="wbte_hide_desc('<?php echo esc_attr( $product->get_id() ); ?>');" style="border:none;margin-top:15px;background:none;">
						<i class="fas fa-times-circle <?php echo esc_attr( ( $on_sale ) ? 'cas-fa-sale' : '' ); ?>"></i>
					</button>
				</div>
			</td>
			<td class="th-center">
				<?php
				$featured         = ( $product->get_featured() ) ? '1' : '0';
				$featured_checked = '';
				if ( '1' === $featured ) {
					$featured_checked = 'checked="checked"';
				}
				?>
				<input type="checkbox" <?php echo esc_attr( ( $has_attributes ) ? 'disabled="disabled"' : '' ); ?> name="featured" <?php echo esc_attr( $featured_checked ); ?> value="<?php echo esc_attr( $product->get_featured() ); ?>">
			</td>
			<td>
				<input type="text" class="input-txt" name="sku" value="<?php echo esc_attr( $this->functions->wbte_get_product_sku( $product ) ); ?>">
			</td>
			<td>
			<input type="text" class="input-txt" name="tags" value="<?php echo esc_attr( $tags ); ?>">
			</td>
			<td>
				<select name="backorder" class="input-select">
					<?php $this->functions->wbte_get_bulk_options( 'backorder', $product->get_backorders() ); ?>
				</select>
			</td>
			<td>
				<select name="instock" class="input-select">
					<?php $this->functions->wbte_get_bulk_options( 'instock', $product->get_stock_status() ); ?>
				</select>
			</td>
			<td>
				<select name="visibility" <?php echo esc_attr( ( $has_attributes ) ? 'disabled="disabled"' : '' ); ?> class="input-select">
					<?php $this->functions->wbte_get_bulk_options( 'visibility', $product->get_catalog_visibility() ); ?>
				</select>
			</td>
			<td>
				<input type="number" class="input-txt" name="weight" value="<?php echo esc_attr( $product->get_weight() ); ?>">
			</td>
			<td>
				<input type="number" class="input-txt" name="_length" value="<?php echo esc_attr( $product->get_length() ); ?>">
			</td>
			<td>
				<input type="number" class="input-txt" name="width" value="<?php echo esc_attr( $product->get_width() ); ?>">
			</td>
			<td>
				<input type="number" class="input-txt" name="height" value="<?php echo esc_attr( $product->get_height() ); ?>">
			</td>
			<td>
				<select name="shipping" class="input-select">
					<?php $this->functions->wbte_get_bulk_options( 'shipping', $product->get_shipping_class_id() ); ?>
				</select>
			</td>
			<td class="th-center">
				<input type="hidden" id="product_img_<?php echo esc_attr( $product->get_id() ); ?>" name="product_img" value="<?php echo esc_attr( $product->get_image_id() ); ?>">
				<a href="#" onclick="openMediaLib('<?php echo esc_js( $product->get_id() ); ?>')">
				<?php
				if ( $product->get_image_id() ) {
					if ( ! $has_attributes ) {
						// Display the main product thumbnail
						echo wp_kses( get_the_post_thumbnail( $product_id, array( '28', '28' ), array( 'class' => 'wbte-thumb-main' ) ), $this->functions->wbte_get_allowed_html() );
					} else {
						// Check if the product is a variation before creating the WC_Product_Variation instance
						if ( $product instanceof WC_Product_Variation ) {
							echo wp_kses( $product->get_image( array( '28', '28' ), array( 'class' => 'wbte-thumb-main' ) ), $this->functions->wbte_get_allowed_html() );
						} else {
							// Fallback for non-variation products with attributes
							echo '<img height="28" src="' . esc_attr( wc_placeholder_img_src() ) . '" class="wbte-thumb-main wp-post-image">';
						}
					}
				} else {
					?>
					<img height="28" src="<?php echo esc_attr( wc_placeholder_img_src() ); ?>" class="wbte-thumb wp-post-image">
					<?php
				}
				?>
				</a>
				
				<a href="#" onclick="wbteRemoveThumbnail( <?php echo esc_js( $product->get_id() ); ?> );"> <i class="fas fa-trash" style="float:right;padding-right:3px;font-size:medium;padding-top:9px;"></i> </a>
				<a href="#" onclick="openMediaLib('<?php echo esc_js( $product->get_id() ); ?>')"><i class="fas fa-edit" style="float:right;padding-right:3px;font-size:medium;"></i></a> 
				</form>
			</td>
		</tr>
		<?php

	}

	/**
	 * Get table footer
	 */
	public function wbte_get_table_footer() {
		?>
		<tfoot>
			<tr>
				<td><?php wp_nonce_field( 'footer_id_ext' ); ?></td>
				<td>
				</td>
				<td colspan="12"></td>
			</tr>
		</tfoot>
		<?php
	}
}


