<?php
/**
 * Bulk Table Editor Settings
 *
 * @package BulkTableEditor/Settings
 */

defined( 'ABSPATH' ) || exit;

/**
 * Settings for API.
 */
if ( class_exists( 'WC_Admin_Settings_WBTE', false ) ) {
	return new WC_Admin_Settings_WBTE();
}

/**
 * Wbte_Setting class
 */
class WC_Admin_Settings_WBTE extends WC_Settings_Page {

	/**
	 * Constructor.
	 */
	public function __construct() {

		$this->id    = 'wbte';
		$this->label = __( 'Bulk Table Editor', 'woo-bulk-table-editor' );

		parent::__construct();
	
		register_setting( 'wbte_options_group', 'wbte_options_settings' );

		//If changes - make sure table is updated
		delete_transient( 'cas_json_products' );
	}

	/**
	 * Get sections
	 *
	 * @return array
	 */
	public function get_sections() {
	
		$sections = array(
			''                   => __( 'General', 'woo-bulk-table-editor' ),
			'wbte_integrations'  => __( 'Integration', 'woo-bulk-table-editor' ),
			'wbte_cron'          => __( 'Cron Job (sales)', 'woo-bulk-table-editor' ),
			'wbte_custom_fields' => __( 'Custom Fields', 'woo-bulk-table-editor' ),
		);
		
		return apply_filters( 'woocommerce_get_sections_' . $this->id, $sections );

	}

	/**
	 * Settings array
	 *
	 * @return array
	 */
	public function get_settings() {

		global $current_section;
		$section  = $current_section;
		$settings = array();

		if ( '' === $section ) {

			$settings = array(

				array(
					'name' => __( 'Bulk Table Editor', 'woo-bulk-table-editor' ),
					'type' => 'title',
					'desc' => __( 'The following options are used to configure Bulk Table Editor', 'woo-bulk-table-editor' ),
					'id'   => 'wbte_general_options',
				),
				
				array(
					'name'     => __( 'Products per page', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Will create a paging function', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_posts_per_page]',
					'type'     => 'number',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Default product category', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select product category or all products', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_product_cat]',
					'type'     => 'button',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Show empty product categories', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'List product categories with 0 products, default lists categories with products', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_categories_all]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Search in selected categories', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Check this to search in the selected categories (text or sku). Default = search in all categories.', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_inlcude_cat_in_search]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Order by (categories)', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Set order by, default is Menu order (same as WooCommerce)', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_categories_orderby]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'options'  => array(
						''           => __( 'Select', 'woo-bulk-table-editor' ),
						'name'       => 'Name',
						'menu_order' => 'Menu order',
						'id'         => 'Id',
						'modified'   => 'Modified',
					)
				),

				array(
					'name'     => __( 'Query products by statuses', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Set the statuses you want to see, default all statuses is shown in the extension', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_product_status]',
					'type'     => 'multiselect',
					'css'      => 'min-width:200px;',
					'options'  => array(
						'publish' => 'Publish',
						'private' => 'Private',
						'draft'   => 'Draft',
						'pending' => 'Pending',
					)
				),

				array(
					'name'     => __( 'Order by (products)', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Set order by, default is Name', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_product_orderby]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'options'  => array(
						'name'       => 'Name',
						'menu_order' => 'Menu order',
						'date'       => 'Date',
						'modified'   => 'Modified',
					)
				),

				array(
					'name'     => __( 'Order (products)', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Set order, default is ASC', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_product_order]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'options'  => array(
						'ASC'  => 'ASC',
						'DESC' => 'DESC',
					)
				),

				array(
					'name'     => __( 'Automatic stock management', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Turns off stock management when stock has null/zero values, defaults sets status to: in stock', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_no_stock_management]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Disable autofocus', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Disables autofocus on input fields and controls in table', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_no_autofocus]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Disable description', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Disables description field, increase of performance if many products/variations', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_disable_description]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Autofocus on date selectors', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'If checked the table date selectors in focus when mouse over', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_date_format_autofocus]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				/*
				array(
					'name'     => __( 'Filter ON SALE by Query (default JS)', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'If many products this function should not be used (no paging so all products is loaded)', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_table_sale_filter]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),*/
				
				array(
					'name'     => __( 'SKU generator length', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Default 3 letters. I.e a product variation: Shirt, Blue, large generates to: shi-blu-lar', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_sku_count]',
					'type'     => 'number',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'SKU delimiter', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Default SKU uses - and creates SHI-BLU-LAR when generating', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_sku_delimiter]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'options'  => array(
						'-'      => __( 'Use default ( - )', 'woo-bulk-table-editor' ),
						'.'      => __( 'Use dot ( . )', 'woo-bulk-table-editor' ),
					),
				),

				array(
					'name'     => __( 'Hide edit options', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Hides: edit product, other values and settings button', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_hide_edit_options]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Hide range filter', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'This option hide range filter in editor home', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_hide_range_filter]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Use Dark Mode', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Check this to enable Dark Mode (style)', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_use_dark_mode]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'type' => 'sectionend',
					'id'   => 'wbte_general_options',
				),
			);

		} elseif ( 'wbte_integrations' === $section ) {

			$settings = array(
				
				array(
					'name' => __( 'Bulk Table Editor', 'woo-bulk-table-editor' ),
					'type' => 'title',
					'desc' => __( 'The following options are used to configure Bulk Table Editor', 'woo-bulk-table-editor' ),
					'id'   => 'wbte_general_integrations',
				),
				
				array(
					'name'     => __( 'Custom price (slug)', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Insert custom slug i.e _wholesaler_price. Recommended to use only a type of price and not i.e a percent value.', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_price_1]',
					'type'     => 'text',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_custom_price',
				),

				array(
					'name'     => __( 'Custom price name', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Your custom column name in the table, keep it short', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_price_1_header]',
					'type'     => 'text',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Integrations (get slug)', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select integration, will only work if plugin is installed', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_integration]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_integration',
					'options'  => array(
						''                      => __( 'Select integration', 'woo-bulk-table-editor' ),
						'_suggested_price'      => __( 'Name your price (suggested price field)', 'woo-bulk-table-editor' ),
						'_cost_of_goods'        => __( 'Cost & Reports (cost of goods field)', 'woo-bulk-table-editor' ),
						'_wc_cog_cost'          => __( 'Cost of goods (cost of goods field)', 'woo-bulk-table-editor' ),
						'_wwp_wholesale_amount' => __( 'Wholesale for WooCommerce (wholesale field)', 'woo-bulk-table-editor' ),
					),
				),

				array(
					'name'     => __( 'Custom price - calculate like sales price', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'If checked it uses price for calculations i.e: normal price - % = custom price. Uncheked it is independent i.e: custom price - % = new custom price', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_price_1_normal_calc]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Show extra column', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Must be checked to show the custom price or SKU on main page', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_price_1_visible]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Show SKU in main page', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'If checked SKU will show as custom column (replaces the custom integration price)', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_use_sku_main_page]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				), 

				array(
					'name'     => __( 'Enable Vendors', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'If checked a column for WooCommerce Vendors and bulk functions is added for setting vendors on products', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_vendor_integration]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),
				
				array(
					'name'     => __( 'Enable Brands', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'If checked a column for WooCommerce Brands and bulk functions is added for setting brands on products', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_use_brands]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'type' => 'sectionend',
					'id'   => 'wbte_general_integrations',
				),

			);
		} elseif ( 'wbte_cron' === $section ) {
			$settings = array(
				
				array(
					'name' => __( 'Bulk Table Editor', 'woo-bulk-table-editor' ),
					'type' => 'title',
					'desc' => __( 'These settings will enable you to run the WooCommerce sales cron job (Action Scheduler) at more frequent intervals', 'woo-bulk-table-editor' ),
					'id'   => 'wbte_general_cron',
				),
				array(
					'name'     => __( 'Use Sales Cron Job', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'If checked, this will hook into the WooCommerce sales cron job with your interval', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_use_sales_cron]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),
				array(
					'name'     => __( 'Cron Interval', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select interval when cron job for sales is executed (e.g. if you use time on your sales)', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_sale_cron_interval]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_interval',
					'options'  => array(
						''      => __( 'Select', 'woo-bulk-table-editor' ),
						'180'   => __( '3 minutes (Use for testing)', 'woo-bulk-table-editor' ),
						'900'   => __( '15 minutes', 'woo-bulk-table-editor' ),
						'1800'  => __( '30 minutes', 'woo-bulk-table-editor' ),
						'3600'  => __( '60 minutes (1 hour)', 'woo-bulk-table-editor' ),
						'7200'  => __( '120 minutes (2 hours)', 'woo-bulk-table-editor' ),
						'14400' => __( '180 minutes (3 hours)', 'woo-bulk-table-editor' ),
						'28800' => __( '360 minutes (6 hours)', 'woo-bulk-table-editor' ),
					),
				),
				array(
					'type' => 'sectionend',
					'id'   => 'wbte_general_cron',
				),

			);
		} elseif ( 'wbte_custom_fields' === $section ) {
			$settings = array(
				
				array(
					'name' => __( 'Bulk Table Editor', 'woo-bulk-table-editor' ),
					'type' => 'title',
					'desc' => __( 'These settings will enable you to add custom product fields in the Attributes & Custom Fields tab.', 'woo-bulk-table-editor' ),
					'id'   => 'wbte_custom_fields_title',
				),

				array(
					'name'     => __( 'Custom Field 1 Label', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Custom field, column label', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_1_name]',
					'type'     => 'text',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Select Field 1', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select custom product field to use', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_1_slug]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_interval',
					'options'  => array_merge( array( '' => __( 'Select Custom Field', 'woo-bulk-table-editor' ) ), $this->cas_get_all_custom_product_fields() ),
				),

				array(
					'name'     => __( 'Custom Field 1 Type', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select type of field e.g. text or number', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_1_type]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_interval',
					'options'  => array(
						''       => __( 'Select', 'woo-bulk-table-editor' ),
						'text'   => __( 'Text', 'woo-bulk-table-editor' ),
						'number' => __( 'Number', 'woo-bulk-table-editor' ),
						'date'   => __( 'Date', 'woo-bulk-table-editor' ),
					),
				),

				array(
					'name'     => __( 'Use Custom Field 1', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Check to add this field to the custom fields table (tab).', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_use_custom_field_1]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Custom Field 2 Label', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Custom field, column label', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_2_name]',
					'type'     => 'text',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Select Field 2', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select custom product field to use', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_2_slug]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_interval',
					'options'  => array_merge( array( '' => __( 'Select Custom Field', 'woo-bulk-table-editor' ) ), $this->cas_get_all_custom_product_fields() ),
				),

				array(
					'name'     => __( 'Custom Field 2 Type', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select type of field e.g. text or number', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_2_type]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_interval',
					'options'  => array(
						''       => __( 'Select', 'woo-bulk-table-editor' ),
						'text'   => __( 'Text', 'woo-bulk-table-editor' ),
						'number' => __( 'Number', 'woo-bulk-table-editor' ),
						'date'   => __( 'Date', 'woo-bulk-table-editor' ),
					),
				),

				array(
					'name'     => __( 'Use Custom Field 2', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Check to add this field to the custom fields table (tab).', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_use_custom_field_2]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Custom Field 3 Label', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Custom field, column label', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_3_name]',
					'type'     => 'text',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Select Field 3', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select custom product field to use', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_3_slug]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_interval',
					'options'  => array_merge( array( '' => __( 'Select Custom Field', 'woo-bulk-table-editor' ) ), $this->cas_get_all_custom_product_fields() ),
				),

				array(
					'name'     => __( 'Custom Field 3 Type', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select type of field e.g. text or number', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_3_type]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_interval',
					'options'  => array(
						''       => __( 'Select', 'woo-bulk-table-editor' ),
						'text'   => __( 'Text', 'woo-bulk-table-editor' ),
						'number' => __( 'Number', 'woo-bulk-table-editor' ),
						'date'   => __( 'Date', 'woo-bulk-table-editor' ),
					),
				),

				array(
					'name'     => __( 'Use Custom Field 3', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Check to add this field to the custom fields table (tab).', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_use_custom_field_3]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Custom Field 4 Label', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Custom field, column label', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_4_name]',
					'type'     => 'text',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Select Field 4', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select custom product field to use', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_4_slug]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_interval',
					'options'  => array_merge( array( '' => __( 'Select Custom Field', 'woo-bulk-table-editor' ) ), $this->cas_get_all_custom_product_fields() ),
				),

				array(
					'name'     => __( 'Custom Field 4 Type', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select type of field e.g. text or number', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_4_type]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_interval',
					'options'  => array(
						''       => __( 'Select', 'woo-bulk-table-editor' ),
						'text'   => __( 'Text', 'woo-bulk-table-editor' ),
						'number' => __( 'Number', 'woo-bulk-table-editor' ),
						'date'   => __( 'Date', 'woo-bulk-table-editor' ),
					),
				),

				array(
					'name'     => __( 'Use Custom Field 4', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Check to add this field to the custom fields table (tab).', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_use_custom_field_4]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Custom Field 5 Label', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Custom field, column label', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_5_name]',
					'type'     => 'text',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Custom Field 5 Slug', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Enter your custom product field slug - as in the above selects but manually enter (for cases where slug is not in dropdowns)', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_5_slug]',
					'type'     => 'text',
					'css'      => 'min-width:200px;',
				),

				array(
					'name'     => __( 'Custom Field 5 Type', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Select type of field e.g. text or number', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_custom_field_5_type]',
					'type'     => 'select',
					'css'      => 'min-width:200px;',
					'class'    => 'wbte_interval',
					'options'  => array(
						''       => __( 'Select', 'woo-bulk-table-editor' ),
						'text'   => __( 'Text', 'woo-bulk-table-editor' ),
						'number' => __( 'Number', 'woo-bulk-table-editor' ),
						'date'   => __( 'Date', 'woo-bulk-table-editor' ),
					),
				),

				array(
					'name'     => __( 'Use Custom Field 5', 'woo-bulk-table-editor' ),
					'desc_tip' => __( 'Check to add this field to the custom fields table (tab).', 'woo-bulk-table-editor' ),
					'id'       => 'wbte_options[wbte_use_custom_field_5]',
					'type'     => 'checkbox',
					'css'      => 'min-width:200px;',
				),

				array(
					'type' => 'sectionend',
					'id'   => 'wbte_custom',
				),
			);
		}

		return apply_filters( 'wc_' . $this->id . '_settings', $settings );

	}
	
	/**
	 * Output the settings
	 *
	 * @since 1.0
	 */
	public function output() {
	
		global $current_section;

		$settings = $this->get_settings( $current_section );
		WC_Admin_Settings::output_fields( $settings );
		echo esc_attr( $this->js_scripts() );
	}
			
	/**
	 * Output JS scripts
	 */
	public function js_scripts() {
		?>
		<script>
		var $ = jQuery;
		jQuery(document).ready(function ( $ ) {
			//$('.wbte_integration').val('');
			$('.wbte_integration').on('change', setIntegration);
		});

		function setIntegration() {
			var selected = $('.wbte_integration option:selected').val();
			$('.wbte_custom_price').val( selected );
		}
		</script>
		<?php
	}

	/**
	 * Get custom fields options
	 */
	private function cas_get_all_custom_product_fields() {
		global $wpdb;

		$meta_keys = $wpdb->get_col(
			$wpdb->prepare(
				"SELECT DISTINCT pm.meta_key
				FROM $wpdb->postmeta pm
				JOIN $wpdb->posts p ON pm.post_id = p.ID
				WHERE pm.meta_key NOT BETWEEN '_' AND '_z'
				AND pm.meta_key NOT LIKE %s
				ORDER BY p.post_modified DESC, pm.meta_key
				LIMIT %d",
				$wpdb->esc_like('_') . '%',
				40
			)
		);

		$custom_fields = array();
	
		foreach ( $meta_keys as $key ) {
			$custom_fields[$key] = $key; // Use meta key as both the key and the label
		}
	
		return $custom_fields;
	}

}

return new WC_Admin_Settings_WBTE();


