<?php
/**
 * Bulk Table Editor templates
 *
 * @package BulkTableEditor/includes
 */

defined( 'ABSPATH' ) || exit;

require_once __DIR__ . '/class-wbte-functions.php';
require_once __DIR__ . '/class-wbte-extended.php';
require_once __DIR__ . '/class-wbte-upsells.php';
require_once __DIR__ . '/class-wbte-custom.php';
require_once __DIR__ . '/class-wbte-category-img.php';

/**
 * Class for templates
 */
class WbteTemplates {

	/**
	 * WbteFunctions
	 *
	 * @var var $wbtefunctions.
	 */
	public $wbtefunctions;

	/**
	 * WbteExtended
	 *
	 * @var var $extended.
	 */
	public $extended;

	/**
	 * Constructor
	 */
	public function __construct() {
		$this->wbtefunctions = new WbteFunctions();
		$this->extended      = new WbteExtended();
	}

	/**
	 * Load the page and contents
	 */
	public function wbte_load() {

		$page_num    = filter_input( 1, 'paged', FILTER_SANITIZE_NUMBER_INT ) ? filter_input( 1, 'paged', FILTER_SANITIZE_NUMBER_INT ) : 1;
		$view        = filter_input( 1, 'view', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$product_obj = $this->wbtefunctions->wbte_get_woo_products();
		$products    = $product_obj['products'];
		$total_pages = $product_obj['max_num_pages'];

		// Set default page if empty
		if ( empty( $view ) ) {
			$view = 'prod';
		}

		?>
		<div class="wbte-main">
		<?php
		$this->wbte_get_bte_header( $view );
		
		switch ( $view ) {
			case '':
			case 'prod':
				$this->wbte_get_table_paging_top( $view, $page_num, $total_pages );
				$this->wbte_get_table( $products, $view, $page_num, $total_pages );
				$this->wbte_get_sticky_footer();
				break;
			case 'ext':
				$this->wbte_get_table_paging_top( $view, $page_num, $total_pages );
				$this->extended->wbte_get_table( $products, $view, $page_num, $total_pages );
				$this->wbte_get_sticky_footer();
				break;
			case 'custom':
				$this->wbte_get_table_paging_top( $view, $page_num, $total_pages );
				Wbte_Custom_Fields::wbte_get_custom_table();
				$this->wbte_get_sticky_footer();
				break;
			case 'cat':
				$this->wbte_get_table_paging_top( $view, $page_num, $total_pages );
				Wbte_Category_Images_Table::wbte_get_custom_table();
				$this->wbte_get_sticky_footer();
				break;
			case 'upsell':
				Wbte_Upsells::wbte_get_upsell_table();
				break;
		}

		$this->wbte_get_mobile_view();

		?>
		</div>
		<?php
		
	}

	/**
	 * Sticky footer
	 */
	public function wbte_get_sticky_footer() {
		
		$view        = filter_input( 1, 'view', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$save_action = 'saveAll()';

		switch ( $view ) {
			case 'ext':
				$save_action = 'saveAllExt()';
				break;
			case 'cat':
				$save_action = 'saveCatImgAll()';
				break;
			case 'custom':
				$save_action = 'wbteSaveCustomTable()';
				break;
		}
		?>
		<footer class="wbte-footer" style="display: flex;">
			<div style="flex: 0 0 37%;text-align:left;padding:4px;color:whitesmoke;">
			<?php 
			$img_file = plugins_url( '../consortia-100.png', __FILE__ );

			/* translators: %s: url to vendor and logo */ 
			printf( esc_html__( '- developed by %1$s%2$s%3$s', 'woo-bulk-table-editor' ), '<a href="https://www.consortia.no/en/" target="_blank">', '<img src="' . esc_attr( $img_file ) . '" class="cas-logo">', '</a>' );
			?>
			</div>
			<div style="flex: 1;text-align:left;">
			<button class="button" id="reset-values-footer" onclick="wbte_reset_table_values();" style="width:150px;"><i class="fas fa-undo"></i> <?php esc_attr_e( 'Undo changes', 'woo-bulk-table-editor' ); ?></button>
			<button type="button" id="saveall-foot" class="button button-primary" onclick="<?php echo esc_js( $save_action ); ?>;" style="width:150px;"><i class="fas fa-database"></i> <?php esc_html_e( 'Save', 'woo-bulk-table-editor' ); ?></button>
			</div>
		</footer>
		<?php
	}

	/**
	 * Check for autofocus
	 */
	public function wbte_get_autofocus_value( $type ) {
		$options         = get_option( 'wbte_options' );
		$mouseover_value = 'this.focus();';

		if ( 'date' === $type ) {
			if ( 'yes' !== $options['wbte_date_format_autofocus'] ) {
				$mouseover_value = '';
			}
		} else {
			if ( 'yes' === $options['wbte_no_autofocus'] ) {
				$mouseover_value = '';
			}
		}

		return $mouseover_value;
	
	}

	/**
	 * Get header and search
	 *
	 * @param var $view page view.
	 */
	public function wbte_get_bte_header( $view ) {

		$tab               = ( ! empty( filter_input( 1, 'tab', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'tab', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : 'main';
		$get_prod_search   = filter_input( 1, 'product_search', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$get_row_search    = filter_input( 1, 'row_search', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$stype             = filter_input( 1, 'stype', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$prod_cat          = filter_input( 1, 'product_cat', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$sales_filter      = filter_input( 1, 'sales_filter', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$categories        = filter_input( 1, 'categories', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$url               = admin_url( 'edit.php?post_type=product&page=wbte-products&sales_filter=' . $sales_filter . '&categories=' . $categories . '&view=' );
		$options           = get_option( 'wbte_options' );
		$hide_edit         = ( isset( $options['wbte_hide_edit_options'] ) && strlen( $options['wbte_hide_edit_options'] ) > 0 ) ? $options['wbte_hide_edit_options'] : 'no';
		$hide_range_filter = ( isset( $options['wbte_hide_range_filter'] ) && strlen( $options['wbte_hide_range_filter'] ) > 0 ) ? $options['wbte_hide_range_filter'] : 'no';
		$auto_date         = 'date';
		$auto_focus        = '';
		$cat_in_search    = ( ! empty( $options['wbte_inlcude_cat_in_search'] ) && 'yes' === $options['wbte_inlcude_cat_in_search'] ) ? true : false;
		
		?>
		<div class="wrap">
			<h1 class="wp-heading-inline"><?php esc_html_e( 'Bulk Table Editor', 'woo-bulk-table-editor' ); ?></h1>
		</div>

		<div class="woocommerce">
			<nav class="nav-tab-wrapper woo-nav-tab-wrapper" id="wbte-tab-nav">
				<a href="<?php echo esc_url( $url . 'prod' ); ?>" class="nav-tab <?php echo ( 'prod' === $view ) ? 'nav-tab-active' : ''; ?>"><i class="fa-solid fa-house"></i> <?php esc_attr_e( 'Home', 'woo-bulk-table-editor' ); ?></a>
				<a href="<?php echo esc_url( $url . 'ext' ); ?>" class="nav-tab <?php echo ( 'ext' === $view ) ? 'nav-tab-active' : ''; ?>"><i class="fa-regular fa-rectangle-list"></i> <?php esc_attr_e( 'Other Values', 'woo-bulk-table-editor' ); ?></a>
				<a href="<?php echo esc_url( $url . 'cat' ); ?>" class="nav-tab <?php echo ( 'cat' === $view ) ? 'nav-tab-active' : ''; ?>"><i class="fa-solid fa-object-group"></i> <?php esc_attr_e( 'Category & Images', 'woo-bulk-table-editor' ); ?></a>
				<a href="<?php echo esc_url( $url . 'custom' ); ?>" class="nav-tab <?php echo ( 'custom' === $view ) ? 'nav-tab-active' : ''; ?>"><i class="fa-solid fa-tag"></i> <?php esc_attr_e( 'Attributes & Custom Fields', 'woo-bulk-table-editor' ); ?></a>
				<a href="<?php echo esc_url( $url . 'upsell' ); ?>" class="nav-tab <?php echo ( 'upsell' === $view ) ? 'nav-tab-active' : ''; ?>"><i class="fa-solid fa-link"></i> <?php esc_attr_e( 'Linked Products', 'woo-bulk-table-editor' ); ?></a>
			</nav>
		</div>


		<?php if ( 'upsell' !== $view ) : ?>	
		<div class="row-container">
			<div style="display:inline-flex;">
				<?php if ( 'prod' === $view && 'yes' !== $hide_range_filter ) : ?>
					<div class="cas-range-box">
						<span style="width:45%;">
						<select id="cas-range-type" onchange="casRangeTypeChanged('','',true);" title="<?php esc_attr_e( 'Select column to filter', 'woo-bulk-table-editor' ); ?>">
							<option value="price"><?php esc_attr_e( 'Price ( Range )', 'woo-bulk-table-editor' ); ?></option>
							<option value="saleprice"><?php esc_attr_e( 'Sale Price ( Range )', 'woo-bulk-table-editor' ); ?></option>
							<option value="stock"><?php esc_attr_e( 'Stock ( Range )', 'woo-bulk-table-editor' ); ?></option>
							<option value="totcol"><?php esc_attr_e( 'Stock Value ( Range )', 'woo-bulk-table-editor' ); ?></option>
						</select>
						<input type="text" id="cas-amount" readonly class="cas-range-amount">
						</span>
						<span id="cas-slider-range" style="margin-bottom:10px;margin-top:10px;width:55%;" title="<?php esc_attr_e( 'Drag the mouse from either side to filter', 'woo-bulk-table-editor' ); ?>"></span>
					</div>
				<?php endif; ?>
				<!-- ACTION BUTTONS -->
				<div class="cas-action-box cas-action-box-inc-buttons" <?php echo esc_attr( ( 'prod' !== $view ) ? 'style=margin-left:0;' : '' ); ?>>
					<?php if ( 'prod' === $view ) : ?>
						<a type="button" href="#" class="button cas-btn-filters" style="margin-right:4px;" id="a-delete-rows" onclick="deleteRows();" title="<?php esc_attr_e( 'Move checked products to trash', 'woo-bulk-table-editor' ); ?>"><i class="fas fa-trash-alt"></i> </a>
						<a type="button" href="#" class="button cas-btn-filters" style="margin-right:4px;" id="a-clear-sales" onclick="clearSales();" title="<?php esc_attr_e( 'Remove sales on visible products', 'woo-bulk-table-editor' ); ?>"><i class="fas fa-eraser"></i> </a>
					<?php endif; ?>
					<a type="button" href="#" class="button" id="reset-values" style="margin-right:4px;" onclick="wbte_reset_table_values();" title="<?php esc_attr_e( 'Undo changes in table (prior to save)', 'woo-bulk-table-editor' ); ?>"><i class="fas fa-undo"></i> </a>
					<?php if ( 'yes' !== $hide_edit ) : ?>
						<a type="button" href="<?php echo esc_url( admin_url( 'admin.php?page=wc-settings&tab=wbte' ) ); ?>" class="button" id="a-settings" style="margin-right:4px;" title="<?php esc_attr_e( 'Go to settings', 'woo-bulk-table-editor' ); ?>"><i class="fas fa-cog"></i> </a> 
					<?php endif; ?>
					<span>
						<form method="post" id="download_products" action="<?php esc_attr_e( admin_url( 'admin-post.php?csv=true&categories=' . $categories ) ); ?>">
							<input type="hidden" name="action" value="return_products_csv_file">
							<button type="submit" name="download_products" class="button" title="<?php esc_attr_e( 'Download product file (.csv)', 'woo-bulk-table-editor' ); ?>">
								<i class="fas fa-cloud-download-alt"></i> 
							</button>
						</form>
					</span>
				</div>
			</div>

			<div class="col-search display-desktop cas-action-search-box">
				<i class="fas fa-info-circle" style="padding-left:1px;"></i><span style="font-size:12px;">
					<?php esc_attr_e( 'Products', 'woo-bulk-table-editor' ); ?>:<span id="wbte-prod-count" style="padding-left:2px;">--</span> | 
					<?php esc_attr_e( 'Variations', 'woo-bulk-table-editor' ); ?>:<span id="wbte-var-count" style="padding-left:2px;">--</span></span>
				<div style="margin-top:4px;">   
				<form name="gs" id="gsSearch" type="get" onsubmit="casSearchSubmit(this);" autocomplete="off">  
					<input type="hidden" name="post_type" value="product">  
					<input type="hidden" name="page" value="wbte-products"> 
					<input type="hidden" name="view" id="form-gs-view" value="<?php echo esc_attr( $view ); ?>">
					<?php if ( $cat_in_search ) : ?>
						<input type="hidden" name="categories" value="<?php echo esc_attr( $categories ); ?>">
					<?php endif; ?>
					<input type="search" value="<?php esc_attr_e( $get_row_search ); ?>" class="input-txt-search" onmouseover="<?php echo esc_js( $this->wbte_get_autofocus_value( $auto_focus ) ); ?>" placeholder="<?php esc_html_e( 'search', 'woo-bulk-table-editor' ); ?><?php echo ' ' . esc_attr( $get_prod_search ); ?>" id="product_search" name="product_search" >
					<button type="submit" id="btnsearch" class="button"><i class="fas fa-search"></i></button><br/>
					<div class="frm-radio">
					<?php esc_attr_e( 'Search in', 'woo-bulk-table-editor' ); ?>
					<input type="radio" name="stype" value="s" <?php echo ( 's' === $stype ) ? 'checked="checked"' : ''; ?>><?php esc_attr_e( 'Text', 'woo-bulk-table-editor' ); ?>
					<input type="radio" name="stype" value="sku" <?php echo ( 'sku' === $stype ) ? 'checked="checked"' : ''; ?>><?php esc_attr_e( 'SKU', 'woo-bulk-table-editor' ); ?>
					<input type="radio" name="stype" value="rows" <?php echo ( 'rows' === $stype || !isset( $stype ) ) ? 'checked="checked"' : ''; ?>><?php esc_attr_e( 'Rows', 'woo-bulk-table-editor' ); ?>
					</div>           
				</form>
				</div>
			</div>
		</div>
		<?php endif; ?>

		<?php
	}

	/**
	 * Get TABLE head info
	 */
	public function wbte_get_table_head_info() {

		$custom_price    = $this->wbtefunctions->wbte_get_custom_price_info();
		$calculate_field = 4;
		$options         = get_option( 'wbte_options' );
		$show_sku        = ! empty( $options[ 'wbte_use_sku_main_page' ] ) ? $options[ 'wbte_use_sku_main_page' ] : 'no';
		$show_vendors    = ! empty( $options[ 'wbte_vendor_integration' ] ) ? $options[ 'wbte_vendor_integration' ] : 'no';
		$show_brands     = ! empty( $options[ 'wbte_use_brands' ] ) ? $options[ 'wbte_use_brands' ] : 'no';

		if ( 'yes' === $custom_price['normal_calc'] ) {
			$calculate_field = 1;
		}

		$auto_date  = 'date';
		$auto_focus = '';

		?>
			<tr class="th-info">
				<th></th>
				<th style="vertical-align: middle;">
					<strong><?php esc_html_e( 'Bulk Editor', 'woo-bulk-table-editor' ); ?></strong><br />
					<span class="th-desc"><?php esc_html_e( 'Fill in values to apply to all visible products. Double click name for description, ID and SKU.', 'woo-bulk-table-editor' ); ?>
					</span>
				</th>
				<th style="vertical-align: top;"><input type="number" name="stock_select" id="stock_select" class="input-txt" onmouseover="<?php echo esc_js( $this->wbte_get_autofocus_value( $auto_focus ) ); ?>"><br/>
					<select name="stock_select_type" id="stock_select_type" class="input-select" onchange="calcSpecial(0,2,'stock_select');">
						<?php $this->wbtefunctions->wbte_get_bulk_options( 'stock' ); ?>
					</select>
				</th>
				<th style="vertical-align: top;"><input type="number" name="price_select" id="price_select" class="input-txt" onmouseover="<?php echo esc_js( $this->wbte_get_autofocus_value( $auto_focus ) ); ?>">
				<br/>
					<select name="price_select_type" id="price_select_type" class="input-select" onchange="calcSpecial(1,3,'price_select');">
						<?php $this->wbtefunctions->wbte_get_bulk_options( 'price' ); ?>
					</select>
				</th>
				<th style="vertical-align: top;"><input type="number" name="sale_price_select" id="sale_price_select" class="input-txt" onmouseover="<?php echo esc_js( $this->wbte_get_autofocus_value( $auto_focus ) ); ?>">
				<br/>
					<select name="sale_price_select_type" id="sale_price_select_type" class="input-select" onchange="calcSpecial(1,4,'sale_price_select');">
						<?php $this->wbtefunctions->wbte_get_bulk_options( 'sale_price' ); ?>
					</select>
				</th>
				<th style="vertical-align: top;">
					<input type="text" class="input-date" id="datep_from" onchange="setSalesDate(this);" onmouseover="<?php echo esc_js( $this->wbte_get_autofocus_value( $auto_date ) ); ?>;" autocomplete="off">
					<input type="time" style="width:100%;float:right;padding-left:5px;" oninput="setSalesTime(this);" id="datep_from_time" value="00:00" min="00:00" max="23:59" pattern="[0-9]{2}:[0-9]{2}">
				</th>
				<th style="vertical-align: top;">
					<input type="text" class="input-date" id="datep_to" onchange="setSalesDate(this);" onmouseover="<?php echo esc_js( $this->wbte_get_autofocus_value( $auto_date ) ); ?>;" autocomplete="off">
					<input type="time" style="width:100%;float:right;padding-left:5px;" oninput="setSalesTime(this);" id="datep_to_time" value="23:59" min="00:00" max="23:59" pattern="[0-9]{2}:[0-9]{2}">
				</th>
				<th><!-- Sale in percent --></th>
				<?php
				if ( 'yes' === $custom_price['active'] && 'no' === $show_sku ) {
					?>
					<th style="vertical-align: top;"><input type="number" name="custom_price_select" id="custom_price_select" class="input-txt" onmouseover="<?php echo esc_js( $this->wbte_get_autofocus_value( $auto_focus ) ); ?>">
					<br/>
						<select name="custom_price_select_type" id="custom_price_select_type" class="input-select" onchange="calcSpecial(<?php echo esc_js( $calculate_field ); ?>,8,'custom_price_select');">
							<?php $this->wbtefunctions->wbte_get_bulk_options( '' ); ?>
						</select>
					</th>
					<?php
				} elseif ( 'yes' === $show_sku ) {
					?>
					<th style="vertical-align:top;"><input type="text" name="sku_select" id="sku_select" class="input-txt" onmouseover="<?php echo esc_js( $this->wbte_get_autofocus_value( $auto_focus ) ); ?>">
					<br/>
						<select name="sku_select_type" id="sku_select_type" class="input-select" onchange="bulkSetValues('sku_select','');">
							<?php $this->wbtefunctions->wbte_get_bulk_options( 'sku' ); ?>
						</select>
					</th>
					<?php
				}
				if ( 'yes' === $show_vendors ) {
					?>
					<th style="vertical-align: bottom;">
					<select class="input-select" name="vendor_select_type" id="vendor_select_type" onchange="bulkSetValues('vendor_select','');">
						<?php $this->wbtefunctions->wbte_get_vendor_select(); ?>
					</select>
					</th>
					<?php
				}
				if ( 'yes' === $show_brands ) {
					?>
					<th style="vertical-align: bottom;">
					<?php $this->wbtefunctions->wbte_get_brands_select(); ?>
					
					<select class="input-select" name="brand_select_type" id="brand_select_type" onchange="bulkSetValues('brand_select','');">
						<?php $this->wbtefunctions->wbte_get_bulk_options( 'brand' ); ?>
					</select>
					</th>
					<?php
				}
				?>
				<th style="vertical-align: top;">
					<button type="button" id="saveall" class="button" onclick='saveAll();' style="width:100%;"><i class="fas fa-database"></i> <?php esc_html_e( 'Save', 'woo-bulk-table-editor' ); ?></button>
				</th>
			</tr>

		<?php
	}

	/**
	 * Get TABLE
	 *
	 */
	public function wbte_get_table( $products, $view, $page_num, $max_pages ) {
		
		?>
		<div class="display-desktop"> 
			<!-- Table -->
			<table class="wp-list-table widefat fixed striped posts" id="wbtetable" style="width:100%;">
				<thead>
					<?php $this->wbte_get_table_head(); ?>
					<?php $this->wbte_get_table_head_info(); ?>
				</thead>
				<tbody id="the-list"> 
					<?php $this->wbte_loop_products( $products ); ?>
				</tbody>
					<?php $this->wbte_get_table_footer(); ?>
			</table>
			<div id="wbte-paging-bottom2" class="cas-div-paging" style="margin-top: 6px;">
				<?php $this->wbtefunctions->wbte_get_pagination( $page_num, $max_pages ); ?>
			</div>

			</div>
		<div>
		<?php
	}

	/**
	 * Loop products
	 */
	private function wbte_loop_products( $products ) {

		$subscriptions_exists = class_exists( 'WC_Product_Subscription' );
	
		foreach ( $products as $product ) {
			if ( $product instanceof WC_Product ) {
				// Determine if the product is a variation or simple product
				$is_variation = $product instanceof WC_Product_Variation;
		
				// Process subscriptions or simple products
				if ( !$is_variation && ( $product->is_type( 'simple' ) || ( $subscriptions_exists && $product->is_type( 'subscription' ) ) ) ) {
					$this->wbte_get_table_row( $product, false );
				// Process variations (including subscription variations)
				} elseif ( $is_variation || ( $subscriptions_exists && $product->is_type( 'variable-subscription' ) ) ) {
					$this->wbte_get_table_row( $product, true );
				} else {
					// Other cases e.g. grouped products
					$this->wbte_get_table_row( $product, false );
				}
			} 
		}
		
	}

	/**
	 * Get TABLE row
	 *
	 * @param var $product object.
	 * @param var $has_attributes bool.
	 */
	public function wbte_get_table_row( $product, $has_attributes ) {

		$product_name       = $product->get_title();
		$product_id         = $product->get_id();
		$custom_price       = $this->wbtefunctions->wbte_get_custom_price_info();
		$date_format        = get_option( 'date_format' );
		$options            = get_option( 'wbte_options' );
		$show_sku           = ( ! empty( $options[ 'wbte_use_sku_main_page' ] ) ) ? $options[ 'wbte_use_sku_main_page' ] : 'no';
		$show_vendors       = ( ! empty( $options[ 'wbte_vendor_integration' ] ) ) ? $options[ 'wbte_vendor_integration' ] : 'no';
		$custom_price_value = '';
		$prev_sale_price    = get_post_meta( $product->get_id(), '_wbte_prev_sale_price', true );
		$sale_time_from     = get_post_meta( $product->get_id(), '_wbte_sale_time_from', true );
		$sale_time_to       = get_post_meta( $product->get_id(), '_wbte_sale_time_to', true );
		$prev_price         = get_post_meta( $product->get_id(), '_wbte_prev_price', true );
		$disable_desc       = ( ! empty( $options[ 'wbte_disable_description' ] ) ) ? $options[ 'wbte_disable_description' ] : 'no';
		$type               = ( $has_attributes ) ? 'var' : 'prod';
		$hide_edit          = ( isset( $options['wbte_hide_edit_options'] ) && strlen( $options['wbte_hide_edit_options'] ) > 0 ) ? $options['wbte_hide_edit_options'] : 'no';
		$tag_ids            = $product->get_tag_ids();
		$show_brands        = ! empty( $options[ 'wbte_use_brands' ] ) ? $options[ 'wbte_use_brands' ] : 'no';

		if ( $has_attributes ) {

			$attributes = $product->get_attributes();

			if ( ! empty( $attributes ) && is_array( $attributes ) ) {
				foreach ( $attributes as $key => $val ) {
					if ( isset( $val ) && is_string( $val ) && strlen( $val ) > 0 ) {
						$product_name .= ', ' . ucfirst( $val );
					}
				}
				$product_name = ucwords( $product_name );
			}

			$product_id = $product->get_parent_id();
			
			if ( 0 === $product_id ) {
				$product_id = $product->get_id();
			} else {
				$tag_ids = wc_get_product( $product_id )->get_tag_ids();
			}
		}

		if ( 'yes' === $custom_price['active'] ) {
			$custom_price_value = get_post_meta( $product->get_id(), $custom_price['price'], true );
		}

		$tags    = $this->wbtefunctions->wbte_get_product_tags( $tag_ids );
		$on_sale = $product->is_on_sale();

		?>
		<tr class="lozad" style="vertical-align: top;" data-type="<?php echo esc_attr( $type ); ?>">
			<td id="<?php echo esc_attr( $product->get_id() ); ?>" class="cas-td-chk">
				<form id="frm_<?php echo esc_attr( $product->get_id() ); ?>_<?php echo esc_attr( wp_rand( 1, 50000 ) ); ?>" method="post">
				<input name="id" value="<?php echo esc_attr( $product->get_id() ); ?>" type="checkbox">
				<input type="hidden" name="tags" value="<?php echo esc_attr( $tags ); ?>">
				<input type="hidden" name="prev_sale_price" value="<?php echo esc_attr( $prev_sale_price ); ?>">
				<input type="hidden" name="prev_price" value="<?php echo esc_attr( $prev_price ); ?>">
				<input type="hidden" name="stock_status" value="<?php echo esc_attr( $product->get_stock_status() ); ?>">
				<input type="hidden" name="brands" value="<?php echo esc_attr( $this->wbtefunctions->wbte_get_product_brands( $product_id, 'slug' ) ); ?>">
				<?php echo wp_kses( (string) Wbte_Custom_Fields::wbte_get_product_attributes_hidden_input( $product ), 'post' ); ?>
			</td>
			<td style="display:flex;width:98%" ondblclick="wbte_show_desc('<?php echo esc_attr( $product->get_id() ); ?>');">
				<?php
				if ( ! $has_attributes ) {
					?>
					<div style="flex:1;width:100%;">
					<input type="text" class="input-txt-name" name="product_name" value="<?php echo esc_html( $product_name ); ?>">
					<input type="hidden" name="name_update" value="true">
					<?php
				} else {
					?>
					<div style="flex:1;width:100%;padding-top:5px;">
					<input type="hidden" name="product_name" value="<?php echo esc_html( $product_name ); ?>">
					<input type="hidden" name="name_update" value="false">
					<?php echo esc_html( $product_name ); ?>
					<?php
				}
				?>
					<?php if ( 'no' === $disable_desc && 'yes' !== $hide_edit ) : ?>
					<div class="wbte-prod-desc-hide" id="wbte_desc_<?php echo esc_attr( $product->get_id() ); ?>" style="padding-top:4px;">
						<textarea id="desc_<?php echo esc_attr( $product->get_id() ); ?>" class="wbte-tiny-editor"><?php echo wp_kses( (string) $product->get_description(), 'post' ); ?></textarea>
					</div>
					<?php endif; ?>
					<ul class="wbte-ul-info-hide" id="wbte_ul_desc_<?php echo esc_attr( $product->get_id() ); ?>">
						<li><?php esc_html_e( 'ID:', 'woo-bulk-table-editor' ); ?> <?php echo esc_attr( $product->get_id() ); ?></li>
						<li><?php esc_html_e( 'SKU:', 'woo-bulk-table-editor' ); ?> <?php echo esc_attr( $product->get_sku() ); ?></li>
					</ul>
				</div>
				<?php if ( 'yes' !== $hide_edit ) : ?>
				<div style="flex: 0;text-align:center;">
					<a href="<?php echo esc_url( get_admin_url() . 'post.php?post=' . $product_id . '&action=edit' ); ?>" target="_new" title="<?php esc_attr_e( 'Edit product', 'woo-bulk-table-editor' ); ?>">
						<i class="fas fa-edit <?php echo esc_attr( ( $on_sale ) ? 'cas-fa-sale' : '' ); ?>"></i>
					</a>
					<button name="btn_desc" id="btn_desc_<?php echo esc_attr( $product->get_id() ); ?>" class="btn-desc-hide" onclick="wbte_hide_desc('<?php echo esc_attr( $product->get_id() ); ?>');" style="border:none;margin-top:15px;background:none;">
						<i class="fas fa-times-circle <?php echo esc_attr( ( $on_sale ) ? 'cas-fa-sale' : '' ); ?>"></i>
					</button>
				</div>
				<?php endif; ?>
				<?php
				if ( $product->get_image_id() ) {
					if ( ! $has_attributes ) {
						// Display the main product thumbnail
						echo wp_kses( get_the_post_thumbnail( $product_id, array( '28', '28' ), array( 'class' => 'wbte-thumb-main' ) ), $this->wbtefunctions->wbte_get_allowed_html() );
					} else {
						// Check if the product is a variation before creating the WC_Product_Variation instance
						if ( $product instanceof WC_Product_Variation ) {
							echo wp_kses( $product->get_image( array( '28', '28' ), array( 'class' => 'wbte-thumb-main' ) ), $this->wbtefunctions->wbte_get_allowed_html() );
						} else {
							// Fallback for non-variation products with attributes
							echo '<img height="28" src="' . esc_attr( wc_placeholder_img_src() ) . '" class="wbte-thumb-main wp-post-image">';
						}
					}
				} else {
					?>
					<img height="28" src="<?php echo esc_attr( wc_placeholder_img_src() ); ?>" class="wbte-thumb-main wp-post-image">
					<?php
				}
				?>
			</td>
			<td>
				<input type="number" name="stock" class="input-txt" <?php echo esc_attr( $product->is_type('external') ? 'readonly="readonly"' : '' ); ?> value="<?php echo esc_attr( $product->get_stock_quantity() ); ?>"> 
			</td>
			<td>
				<input type="number" step="any" name="price" class="input-txt" value="<?php echo esc_attr( $product->get_regular_price() ); ?>"> 
			</td> 
			<td>
				<input type="number" step="any" name="saleprice" class="input-txt" value="<?php echo esc_attr( $product->get_sale_price() ); ?>" onchange="calcPercent(this);"> 
			</td>   
			<td>
			<?php
			$sale_from_value    = '';
			$sale_from_utc_date = '';
			if ( $product->get_date_on_sale_from() ) {
				$sale_from_value    = date_format( $product->get_date_on_sale_from(), $date_format );
				$sale_from_utc_date = date_format( $product->get_date_on_sale_from(), 'Y-m-d' ); 
			};
			?>
				<input type="text" style="width:53%;float:left;margin:0;" name="salefrom" data-utc-date="<?php echo esc_attr( $sale_from_utc_date ); ?>" autocomplete="off" class="input-date" value="<?php echo esc_attr( $sale_from_value ); ?>"> 
				<input type="time" style="width:46%;float:right;margin:0;padding-left:2px;" name="salefrom_time" value="<?php echo ( ! empty( $sale_time_from ) ) ? esc_attr( $sale_time_from ) : '00:00'; ?>" min="00:00" max="23:59" pattern="[0-9]{2}:[0-9]{2}">
			</td>
			<td>
			<?php
			$sale_to_value    = '';
			$sale_to_utc_date = '';
			if ( $product->get_date_on_sale_to() ) {
				$sale_to_value    = date_format( $product->get_date_on_sale_to(), $date_format );
				$sale_to_utc_date = date_format( $product->get_date_on_sale_to(), 'Y-m-d' ); 
			}
			?>
				<input type="text" style="width:53%;float:left;margin:0;" name="saleto" data-utc-date="<?php echo esc_attr( $sale_to_utc_date ); ?>" autocomplete="off" class="input-date" value="<?php echo esc_attr( $sale_to_value ); ?>"> 
				<input type="time" style="width:46%;float:right;margin:0;padding-left:2px;" name="saleto_time" value="<?php echo ( ! empty( $sale_time_to ) ) ? esc_attr( $sale_time_to ) : '23:59'; ?>" min="00:00" max="23:59" pattern="[0-9]{2}:[0-9]{2}">
			</td>
			<td><input type="text" name="salepercent" class="input-txt" value="" readonly="readonly" style="direction:rtl;"></td>
			<?php
			if ( 'yes' === $custom_price['active'] && 'no' === $show_sku ) {
				?>
				<td>
				<input type="number" step="any" name="customprice" class="input-txt" value="<?php echo esc_attr( $custom_price_value ); ?>"> 
				</td>
				<?php
			} elseif ( 'yes' === $show_sku ) {
				?>
				<td>
					<input type="text" class="input-txt" name="sku" value="<?php echo esc_attr( $this->wbtefunctions->wbte_get_product_sku( $product ) ); ?>">
				</td>
				<?php
			}
			if ( 'yes' === $show_vendors ) {
				?>
				<td>
					<?php $this->wbtefunctions->wbte_get_vendor_selected( $product_id ); ?>
				</td>
				<?php
			}
			if ( 'yes' === $show_brands ) {
				$brands       = wp_get_post_terms( $product_id, 'product_brand' );
				$brands_slugs = $this->wbtefunctions->wbte_get_product_brands( $product->get_id(), 'slug' );
				?>
				<td>
					<input type="hidden" name="brand_input_hidden" value="<?php echo esc_attr( $brands_slugs ); ?>">
					<input type="txt" name="brand_input" class="input-txt cas-brand-input" style="display:none;" readonly="readonly" value="<?php echo esc_attr( $brands_slugs ); ?>">
					<div class="cas-td-scroll-brand" id="cas-brand-div-<?php echo esc_attr( $product_id ); ?>" title="<?php esc_attr_e( 'Click to delete (if variable - removes from parent)', 'woo-bulk-table-editor' ); ?>">
					<?php 
					if ( $brands ) {
						$brands_list = '';
						foreach ( $brands as $_brand ) {
							$del_id       = $_brand->term_id . '_' . $product_id;
							$brands_list .= '<div id="link_' . $del_id . '"><a href="#" onclick="wbteRemoveBrand(' . esc_js( $_brand->term_id ) . ',' . esc_js( $product_id ) . ');"><i class="fas fa-check-square"></i> ' . esc_attr( $_brand->name ) . '</a></div>';
						}
						echo wp_kses( $brands_list, $this->wbtefunctions->wbte_get_allowed_html() );
					}
					?>
					</div>
				</td>
				<?php
			}
			$stock_value = '';
			if ( null !== $product->get_stock_quantity() && $product->get_stock_quantity() > 0 && 
				 null !== $product->get_regular_price() && $product->get_regular_price() > 0 ) {
				$stock_value = wc_format_decimal( $product->get_stock_quantity() * $product->get_regular_price() );
			}
			?>
			<td>
				<input type="number" name="totcol" class="input-txt" value="<?php echo esc_attr( $stock_value ); ?>" readonly="readonly" style="direction:rtl;"> 
				</form>
			</td>
		</tr>
		<?php
	}

	/**
	 * No row found
	 */
	public function wbte_get_no_row_found() {
		
		$custom_price = $this->wbtefunctions->wbte_get_custom_price_info();
		$colspan_val  = 8;
		
		if ( 'yes' === $custom_price['active'] ) {
			$colspan_val = 9;
		}
		?>
			<tr><td colspan="<?php echo esc_attr( $colspan_val ); ?>"><strong><?php esc_html_e( 'No products found', 'woo-bulk-table-editor' ); ?></strong></td></tr>
		<?php

	}

	/**
	 * Get table footer
	 */
	public function wbte_get_table_footer() {
		
		$custom_price = $this->wbtefunctions->wbte_get_custom_price_info();
		$options      = get_option( 'wbte_options' );
		$show_vendors = ( ! empty( $options[ 'wbte_vendor_integration' ] ) ) ? $options[ 'wbte_vendor_integration' ] : 'no';
		$show_brands  = ! empty( $options[ 'wbte_use_brands' ] ) ? $options[ 'wbte_use_brands' ] : 'no';
		$colspan_val  = 8;
		
		if ( 'yes' === $custom_price['active'] ) {
			$colspan_val++;
		}

		if ( 'yes' === $show_vendors ) {
			$colspan_val++;
		}

		if ( 'yes' === $show_brands ) {
			$colspan_val++;
		}
		?>
		<tfoot>
			<tr>
				<td><?php wp_nonce_field( 'footer_id' ); ?></td>
				<td><input type="hidden" id="msg-confirm-delete" value="<?php esc_attr_e( 'Move checked items to trash?', 'woo-bulk-table-editor' ); ?>"></td>
				<td><span id="tbl-total-s" class="span-txt-center"></span></td>
				<td></td>
				<td></td>
				<td></td>
				<td></td>
			<?php
			if ( 'yes' === $custom_price['active'] ) {
				?>
				<td></td>
				<?php
			}
			if ( 'yes' === $show_vendors ) {
				?>
				<td></td>
				<?php
			}
			if ( 'yes' === $show_brands ) {
				?>
				<td></td>
				<?php
			}
			?>
				<td></td>
				<td><span id="tbl-total-f" class="span-txt"></span></td>
			</tr>
			<tr class="th-info">
				<td colspan="<?php echo esc_attr( $colspan_val ); ?>">
				</td>
				<td>
				
				</td>
			</tr>
		</tfoot>
		<?php
	}

	/**
	 * Get paging above table
	 *
	 * @param var $view the view.
	 */
	public function wbte_get_table_paging_top( $view, $page_num, $max_pages ) {

		$options        = get_option( 'wbte_options' );
		$get_prod_cat   = ( ! empty( filter_input( 1, 'product_cat', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'product_cat', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : get_option( 'wbte_options' )['wbte_product_cat'];
		$prod_search    = filter_input( 1, 'product_search', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
		$hide_edit      = ( isset( $options['wbte_hide_edit_options'] ) && strlen( $options['wbte_hide_edit_options'] ) > 0 ) ? $options['wbte_hide_edit_options'] : 'no';
		$filter         = ( ! empty( filter_input( 1, 'sales_filter', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'sales_filter', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		$view           = ! empty( $view ) ? $view : 'prod';
		$form_submitted = filter_input( 1, 'form_submitted', FILTER_VALIDATE_INT );
		$categories     = ! empty( filter_input( 1, 'categories', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ? filter_input( 1, 'categories', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';

		if ( ! $form_submitted && empty( $categories ) ) {
			$categories = ! empty( $get_prod_cat ) ? get_term_by( 'id', $get_prod_cat, 'product_cat' )->slug : '';
		}

		if ( ! empty( $prod_search ) ) {
			$get_prod_cat = '';
		}
		
		?>
		<div class="display-desktop cas-action-box">
			<div class="col-category"> 
				<form method="get" action="edit.php" name="gc" id="cas-form-category-filter-<?php echo esc_attr( $view ); ?>" onsubmit="cas_get_categories_home(this,'cat-<?php echo esc_attr( $view ); ?>','categories');">
					<input type="hidden" name="post_type" value="product">	
					<input type="hidden" name="page" value="wbte-products">
					<input type="hidden" name="view" value="<?php echo esc_attr( $view ); ?>">
					<input type="hidden" name="categories" value="<?php echo esc_attr( $categories ); ?>">
					<input type="hidden" name="sales_filter" value="<?php echo esc_attr( $filter ); ?>">
					<input type="hidden" name="form_submitted" value="1">
					<?php $this->wbtefunctions->wbte_create_category_dropdown( 'wbte-category-filter-' . $view, 'cat-' . $view ); ?>
					<button type="submit" class="button" title="<?php esc_attr_e( 'Get Category Products', 'woo-bulk-table-editor' ); ?>"> <i class="fa-solid fa-filter"></i> </button>
				</form>

				<div style="padding-right:2px;padding-left:2px;">
					<?php $this->wbtefunctions->wbte_get_sales_products_select(); ?>
				</div>
				<div style="padding-right:2px;">
					<?php $this->wbtefunctions->wbte_get_tags_filter(); ?>
				</div>
				<div style="padding-right:2px;">
					<?php $this->wbtefunctions->wbte_get_stock_status_filter(); ?>
				</div>
				<div style="padding-right:2px;">
					<form name="attribute-filter" id="attribute-filter" onchange="wbteFilterSales();">
					<?php echo wp_kses( $this->wbtefunctions->wbte_get_attributes_dropdown_select( 'wbte_attribute_filter' ), $this->wbtefunctions->wbte_get_allowed_html() ); ?>
					</form>
				</div>
				<?php if ( class_exists( 'WC_Brands' ) ) : ?>
					<div style="padding-right:2px;">
						<?php $this->wbtefunctions->wbte_get_brands_filter(); ?>
					</div>
				<?php endif; ?>
				<div class="cas-col-btns" style="align-items:center; padding-top:0px; padding-right:2px;">
					<a type="button" href="#" class="button cas-btn-filters" id="reset-values" onclick="wbte_reset_filters();" title="<?php esc_attr_e( 'Reset all filters', 'woo-bulk-table-editor' ); ?>"><i class="fas fa-filter"></i> <?php esc_attr_e( 'Reset', 'woo-bulk-table-editor' ); ?></a>
				</div>

			</div>

			<div id="wbte-paging-top2" class="cas-div-paging">
				<?php $this->wbtefunctions->wbte_get_pagination( $page_num, $max_pages ); ?>
			</div>

			
		</div>

		<div style="height:10px;margin-top:2px;margin-bottom:6px;">
			<div class="wbte-saving" id="wbte-saving" style="max-width:100%;">
				<progress id="pbar-saving" class="pbar-saving" value="0" max="100" style="height:10px;"></progress>
			</div>
			<div class="wbte-saving" id="wbte-loading-page" style="max-width:100%;">
				<progress id="pbar-loading-page" class="pbar-saving" value="0" max="100" style="height:10px;"></progress>
			</div>
		</div>

		<?php
	}

	/**
	 * Get TABLE head (first row - headings)
	 */
	public function wbte_get_table_head() {

		$custom_price = $this->wbtefunctions->wbte_get_custom_price_info();
		$options      = get_option( 'wbte_options' );
		$show_sku     = ! empty( $options[ 'wbte_use_sku_main_page' ] ) ? $options[ 'wbte_use_sku_main_page' ] : 'no';
		$show_vendors = ! empty( $options[ 'wbte_vendor_integration' ] ) ? $options[ 'wbte_vendor_integration' ] : 'no';
		$disable_desc = ! empty( $options[ 'wbte_disable_description' ] ) ? $options[ 'wbte_disable_description' ] : 'no';
		$desc_label   = ( 'no' === $disable_desc ) ? __( 'Description ID SKU', 'woo-bulk-table-editor' ) : __( 'Show ID & SKU', 'woo-bulk-table-editor' );
		$show_brands  = ! empty( $options[ 'wbte_use_brands' ] ) ? $options[ 'wbte_use_brands' ] : 'no';

		$sale_label = __( 'If using date and time, make sure Cron Job / Action Scheduler is set up in settings.', 'woo-bulk-table-editor' );
		?>
		<tr>
			<th class="th-first"><input type="checkbox" id="checkall" onclick="checkVisible();"></th>
			<th style="width:22%;">
				<a href="#" onclick="sort(1,'text');"><i id="s1" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Name', 'woo-bulk-table-editor' ); ?></a>
				<input type="checkbox" id="wbte-chk-show-desc" onclick="wbte_show_hide_desc();"> <?php echo esc_attr( $desc_label ); ?>
			</th>
			<th><a class="cas-th-stock" href="#" onclick="sort(2,'number');"><i id="s2" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Stock', 'woo-bulk-table-editor' ); ?></a></th>
			<th><a class="cas-th-price" href="#" onclick="sort(3,'number');"><i id="s3" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Price', 'woo-bulk-table-editor' ); ?></a></th>
			<th><a class="cas-th-saleprice" href="#" onclick="sort(4,'number');"><i id="s4" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Sale price', 'woo-bulk-table-editor' ); ?></a></th>
			<th class="th-center" style="width: 170px;"><a href="#" onclick="sort(5,'date');"><i id="s5" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Sale start date', 'woo-bulk-table-editor' ); ?> <i title="<?php echo esc_attr( $sale_label ); ?>" class="fa-solid fa-circle-info"></i></a></th>
			<th class="th-center" style="width: 170px;"><a href="#" onclick="sort(6,'date');"><i id="s6" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Sale end date', 'woo-bulk-table-editor' ); ?> <i title="<?php echo esc_attr( $sale_label ); ?>" class="fa-solid fa-circle-info"></i></a></th>
			<th style="width:60px;" class="th-center"><a href="#" onclick="sort(7,'percent');"><i id="s7" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Sale %', 'woo-bulk-table-editor' ); ?></a></th>
		<?php
		$stock_sort = 8;
		if ( 'yes' === $custom_price['active'] && 'no' === $show_sku ) {
			$stock_sort++;	
			?>
			<th class="th-center"><a href="#" onclick="sort(8,'number');"><i id="s8" class="fas fa-sort cas-i-space"></i><?php echo esc_html( $custom_price['name'] ); ?></a></th>
			<?php
		} elseif ( 'yes' === $show_sku ) {
			$stock_sort++;
			?>
			<th class="th-center"><a href="#" onclick="sort(8,'text');"><i id="s8" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'SKU', 'woo-bulk-table-editor' ); ?></a></th>
			<?php
		} 
		if ( 'yes' === $show_vendors ) {
			$stock_sort++;
			?>
			<th class="th-center"><?php esc_html_e( 'Vendor', 'woo-bulk-table-editor' ); ?></th>
			<?php
		}
		if ( 'yes' === $show_brands ) {
			$stock_sort++;
			?>
			<th class="th-center"><?php esc_html_e( 'Brands', 'woo-bulk-table-editor' ); ?></th>
			<?php
		}
		?>
		<th class="th-center"><a href="#" onclick="sort(<?php echo esc_js( $stock_sort ); ?>,'number');"><i id="s<?php echo esc_js( $stock_sort ); ?>" class="fas fa-sort cas-i-space"></i><?php esc_html_e( 'Stock value', 'woo-bulk-table-editor' ); ?></a></th>
		</tr>

		<?php
	}

	/**
	 * Get mobile view
	 */
	public function wbte_get_mobile_view() {

		?>
		<div class="display-mobile">
			<div class="updated woocommerce-info display-mobile" style="width: 98%;">
				<?php esc_html_e( 'This plugin currently does not support mobile views. ', 'woo-bulk-table-editor' ); ?>
				<?php esc_html_e( 'Please use screen size larger than 700px', 'woo-bulk-table-editor' ); ?>
			</div>
		</div>
		<?php

	}


}
