<?php
/**
 * Bulk Table Editor
 *
 * @package BulkTableEditor/includes
 */

defined( 'ABSPATH' ) || exit;

/**
 * Class for upsells
 */
class Wbte_Upsells {


	/**
	 * Get categories
	 */
	public static function wbte_upsell_get_categories() {
		
		// Get product categories
		$args = array(
			'taxonomy'     => 'product_cat',
			'hide_empty'   => true,
			'hierarchical' => 1,
			'pad_counts'   => 1,
		);

		return get_terms( $args );
	}

	/**
	 * Create dropdown categories
	 */
	public static function wbte_create_category_dropdown( $id, $name = '' ) {
		$options    = get_option( 'wbte_options' );
		$hide_empty = ! empty( $options['wbte_categories_all'] ) && 'yes' ===  $options['wbte_categories_all'] ? 0 : 1;
		$order_by   = ! empty( $options['wbte_categories_orderby'] ) ? $options['wbte_categories_orderby'] : 'menu_order';

		$args = array(
			'taxonomy'          => 'product_cat',
			'hide_empty'        => $hide_empty,
			'hierarchical'      => 1,
			'pad_counts'        => 1,
			'show_count'        => 1,
			'value_field'       => 'slug',
			'orderby'           => $order_by,
			'option_none_value' => '',
			'show_option_none'  => __( 'Select Categories', 'woo-bulk-table-editor' ),
			'id'                => $id,
			'name'              => $name,
		);

		wp_dropdown_categories( $args );
	}

	/**
	 * Get products
	 */
	public static function wbte_get_main_products( $categories, $search = '' ) {
		
		$args = array(
			'orderby'  => 'name',
			'order'    => 'ASC',
			'tax_query' => array(
				array(
					'taxonomy' => 'product_cat',
					'field'    => 'slug',
					'terms'    => explode( ',', $categories ),
					'operator' => 'IN',
				),
			),
			'limit' => -1,
		);

		if ( strlen( $search ) > 0 ) {
			
			$args = array(
				'orderby'  => 'name',
				'order'    => 'ASC',
				's'        => $search,
				'limit'    => -1,
			);
			
		}
		
		return wc_get_products( $args );
	}

	/**
	 * Get brands (slugs) on a product
	 */
	public static function wbte_get_product_brands( $product_id, $type = 'slug' ) {
		
		$list = '';

		if ( empty( $product_id ) ) {
			return $list;
		}
		
		if ( class_exists( 'WC_Brands' ) ) {
			$brands = wp_get_post_terms( $product_id, 'product_brand' );
			if ( ! empty( $brands ) ) {
				foreach ( $brands as $brand ) {
					if ( 'name' === $type ) {
						$list .= $brand->name . ', ';
					} else {
						$list .= $brand->slug . ', ';
					}
				}
				$list = rtrim( $list, ', ' );
			}
		}

		return $list;
	}

	/**
	 * Get upsell table
	 */
	public static function wbte_get_upsell_table() {
		?>
		<div class="wbte-flex-container" style="display: flex; justify-content: space-between;">
		<?php
			wp_nonce_field( 'wbte_linked_id' );
			self::wbte_display_main_products_table();
			self::wbte_display_upsell_products_table();
		?>
		</div>
		<?php
	}

	/**
	 * Display the main products table.
	 */
	public static function wbte_display_main_products_table() {

		$search_str  = ( ! empty( filter_input( 1, 's', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 's', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		$search_str2 = ( ! empty( filter_input( 1, 's2', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 's2', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		$categories  = ( ! empty( filter_input( 1, 'categories', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'categories', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		$products    = ( ! empty( $categories ) || strlen( $search_str ) > 0 ) ? self::wbte_get_main_products( $categories, $search_str ) : '';
		?>
		<!-- Main Products Table -->
		<div class="wbte-main-products" style="flex: 0 0 63%;padding-right:10px;">
			<h2><?php esc_attr_e( 'Products', 'woo-bulk-table-editor' ); ?></h2>

			<div class="wbte-filters" style="margin-bottom:4px;">

				<!-- Product Category Filter -->
				<div class="wbte-category-filter-style" style="display:flex;">
				<div style="flex-grow:5;">
				<form method="get" action="edit.php" id="cas-form-category-filter" onsubmit="cas_upsell_get_category(this,'cat1','categories');">
					<input type="hidden" name="post_type" value="product">	
					<input type="hidden" name="page" value="wbte-products">
					<input type="hidden" name="view" value="upsell">
					<input type="hidden" name="categories" value="<?php echo esc_attr( $categories ); ?>">
					<input type="hidden" name="categories2" value="">
					<?php self::wbte_create_category_dropdown( 'wbte-category-filter', 'cat1' ); ?>
					<input type="hidden" name="s2" value="<?php echo esc_attr( $search_str2 ); ?>">
					<button type="submit" class="button button-primary" title="<?php esc_attr_e( 'Get products from categories', 'woo-bulk-table-editor' ); ?>"> <i class="fa-solid fa-filter"></i> <?php esc_attr_e( 'Get Products', 'woo-bulk-table-editor' ); ?> </button>
					<a href="<?php echo esc_url( admin_url( 'edit.php?post_type=product&page=wbte-products&view=upsell' ) ); ?>" class="button" title="<?php esc_attr_e( 'Resets all filters', 'woo-bulk-table-editor' ); ?>"><i class="fa-solid fa-rotate-left"></i> <?php esc_attr_e( 'Reset', 'woo-bulk-table-editor' ); ?></a>
				</form>
				
				</div>
				<div style="display: inline-block;text-align:right;">
					<form method="get" id="wbte_upsell_search_filter" autocomplete="off" onsubmit="wbte_SearchSubmit(this);">
					<input type="hidden" name="post_type" value="product">	
					<input type="hidden" name="page" value="wbte-products">
					<input type="hidden" name="view" value="upsell">
					<input type="hidden" name="categories2" value="">
					<div id="cas-on-off-switch-div" class="cas-on-off-switch" title="<?php esc_attr_e( 'Filter in table rows(left) or Search (database)', 'woo-bulk-categories' ); ?>">
						<input type="checkbox" class="cas-toggle-input" id="cas-search-toggle" <?php echo esc_attr( strlen( $search_str ) > 0 ? 'checked="checked"' : '' ); ?>>
						<label class="cas-toggle-label" for="cas-search-toggle"></label>
					</div>
					<input type="hidden" name="s2" value="<?php echo esc_attr( $search_str2 ); ?>">
					<input type="search" id="wbte_s" onmouseover="this.focus();" name="s" class="wbc-txt-search">
					<button type="submit" id="wbte_search_submit" class="button"><i class="fas fa-search"></i> <?php esc_attr_e( 'Search', 'woo-bulk-categories' ); ?></button>
					
					</form>
				</div>
			</div>

			</div>

			<table id="wbte-linked-product-table" class="wp-list-table widefat fixed striped">
				<thead>
					<tr>
						<th style="width:35px;"><input type="checkbox" onclick="wbteCheckAllRows('wbte-linked-product-table', 'productId', this);"> <i class="fa-solid fa-circle-question" title="<?php esc_attr_e( 'Click to select all / none', 'woo-bulk-table-editor' ); ?>"></i></th>
						<th style="width:60px;"><?php esc_attr_e( 'ID', 'woo-bulk-table-editor' ); ?></th>
						<th style="width:30px;"></th>
						<th><?php esc_attr_e( 'Name', 'woo-bulk-table-editor' ); ?> <i class="fa-solid fa-circle-question" title="<?php esc_attr_e( 'Click checkbox or in a row to select products', 'woo-bulk-table-editor' ); ?>"></i></th>
						<th style="width:70px;"><?php esc_attr_e( 'Type', 'woo-bulk-table-editor' ); ?></th>
						<th style="width:140px;"><?php esc_attr_e( 'Price', 'woo-bulk-table-editor' ); ?></th>
						<th><?php esc_attr_e( 'Upsells', 'woo-bulk-table-editor' ); ?> <i class="fa-solid fa-circle-question" title="<?php esc_attr_e( 'Upsells are products which you recommend instead of the currently viewed product, for example, products that are more profitable or better quality or more expensive.', 'woo-bulk-table-editor' ); ?>"></i></th>
						<th><?php esc_attr_e( 'Cross-sells', 'woo-bulk-table-editor' ); ?> <i class="fa-solid fa-circle-question" title="<?php esc_attr_e( 'Cross-sells are products which you promote in the cart, based on the current product.', 'woo-bulk-table-editor' ); ?>"></i></th>
					</tr>
				</thead>
				<tbody>
					<?php 
					if ( $products ) : 
						foreach ( $products as $product ) {
							?>
							<tr>
							<td><input type="checkbox" name="productId" id="<?php echo esc_attr( $product->get_id() ); ?>"></td>
							<td><a href="<?php echo esc_url( get_admin_url() . 'post.php?post=' . $product->get_id() . '&action=edit' ); ?>" target="new"><?php echo esc_attr( $product->get_id() ); ?></a></td>
							<td>
							<?php
							if ( $product->get_image_id() ) {
								echo wp_kses( get_the_post_thumbnail( $product->get_id(), array( '28', '28' ), array( 'class' => 'wbte-thumb-main' ) ), 'post' );
							} else {
								?>
								<img height="28" src="<?php echo esc_attr( wc_placeholder_img_src() ); ?>" class="wbte-thumb-main wp-post-image">
								<?php
							}
							?>
							</td>
							
							<td>
								<a href="<?php echo esc_url( get_admin_url() . 'post.php?post=' . $product->get_id() . '&action=edit' ); ?>" target="new"><?php echo esc_attr( $product->get_name() ); ?></a><br/>
								<small><?php esc_attr_e( 'SKU:', 'woo-bulk-table-editor' ); ?> <?php echo esc_attr( $product->get_sku() ); ?></small>
								<?php 
								if ( class_exists( 'WC_Brands' ) ) {
									$brands = self::wbte_get_product_brands( $product->get_id(), 'name' );
									if ( ! empty( $brands ) ) {
										?>
											<br/><small><?php esc_attr_e( 'Brand:', 'woo-bulk-table-editor' ); ?> <?php echo esc_attr( $brands ); ?></small>
										<?php
									}
								}
								?>
							</td>
							<td><?php echo esc_attr( ucfirst( $product->get_type() ) ); ?></td>
							<td>
								<?php echo wp_kses( ( $product->is_on_sale() ) ? '<s>' . wc_price( $product->get_regular_price() ) . '</s> ' . wc_price( $product->get_price() ) : wc_price( $product->get_price() ), 'post' ); ?>
							</td>
							<td>
								<div class="cas-td-scroll">
								<?php 
								foreach ( $product->get_upsell_ids() as $upsell_id ) { 
									$product_up = wc_get_product( $upsell_id );
									?>
									<span id="up_<?php echo esc_attr( $product->get_id() . '_' . $product_up->get_id() ); ?>"><i class="fa-solid fa-square-check" title="<?php esc_attr_e( 'Click to remove', 'woo-bulk-table-editor' ); ?>" onclick="wbteRemoveCrossProduct(<?php echo esc_attr( $product->get_id() ); ?>, <?php echo esc_attr( $product_up->get_id() ); ?>,'up');"></i> <?php echo esc_attr( $product_up->get_name() ); ?> <sup class="cas-sup-price"><?php echo wp_kses( wc_price( $product_up->get_price() ), 'post' ); ?></sup></span><br/>
									<?php
								}
								?>
								</div>
							</td>
							<td>
								<div class="cas-td-scroll">
								<?php 
								foreach ( $product->get_cross_sell_ids() as $cross_id ) { 
									$product_cross = wc_get_product( $cross_id );
									?>
									<span id="cross_<?php echo esc_attr( $product->get_id() . '_' . $product_cross->get_id() ); ?>"><i class="fa-solid fa-square-check" title="<?php esc_attr_e( 'Click to remove', 'woo-bulk-table-editor' ); ?>" onclick="wbteRemoveCrossProduct(<?php echo esc_attr( $product->get_id() ); ?>, <?php echo esc_attr( $product_cross->get_id() ); ?>,'cross');"></i> <?php echo esc_attr( $product_cross->get_name() ); ?> <sup class="cas-sup-price"><?php echo wp_kses( wc_price( $product_cross->get_price() ), 'post' ); ?></sup></span><br/>
									<?php
								}
								?>
								</div>
							</td>
							</tr>
							<?php 
						}
						?>
					<?php else : ?>
						<tr><td colspan="8"><?php esc_attr_e( 'No products found! Select categories or search for products.', 'woo-bulk-table-editor' ); ?></td></tr>	
					<?php endif; ?>
				</tbody>
			</table>
		</div>
		<?php
	}

	/**
	 * Display the upsell products table.
	 */
	public static function wbte_display_upsell_products_table() {
		
		$search_str  = ( ! empty( filter_input( 1, 's', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 's', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		$search_str2 = ( ! empty( filter_input( 1, 's2', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 's2', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		$categories  = ( ! empty( filter_input( 1, 'categories2', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) ) ) ? filter_input( 1, 'categories2', FILTER_SANITIZE_FULL_SPECIAL_CHARS ) : '';
		$products    = ( ! empty( $categories ) || strlen( $search_str2 ) > 0 ) ? self::wbte_get_main_products( $categories, $search_str2 ) : '';
		?>
		<!-- Upsell Products Table -->
		<div class="wbte-upsell-products">
			<h2><?php esc_attr_e( 'Select Upsell And Cross-Sells Products', 'woo-bulk-table-editor' ); ?></h2>

			<div class="wbte-filters" style="margin-bottom:4px;">
			<div style="margin-top: 10px; margin-bottom:20px;">
			
			<button type="button" class="button" onclick="wbteLinkProducts('up');" title="<?php esc_attr_e( 'Upsell checked products', 'woo-bulk-table-editor' ); ?>"> <i class="fa-solid fa-arrow-up"></i> <?php esc_attr_e( 'Upsell', 'woo-bulk-table-editor' ); ?></button>
			<button type="button" class="button" onclick="wbteLinkProducts('cross');" title="<?php esc_attr_e( 'Cross-Sell checked products', 'woo-bulk-table-editor' ); ?>"> <i class="fa-solid fa-arrows-split-up-and-left"></i> <?php esc_attr_e( 'Cross-Sell', 'woo-bulk-table-editor' ); ?></button>
			<button type="button" class="button" onclick="wbteLinkProducts('up-cross');" title="<?php esc_attr_e( 'Upsell and Cross-Sell checked products', 'woo-bulk-table-editor' ); ?>"> <i class="fa-solid fa-arrows-up-down-left-right"></i> <?php esc_attr_e( 'Upsell and Cross-Sell', 'woo-bulk-table-editor' ); ?></button>
			
			</div>
				
			<div style="text-align:right;margin-bottom:10px;">
				<form method="get" id="wbte_upsell_search_filter2" autocomplete="off" onsubmit="wbte_SearchSubmit2(this);">
				<input type="hidden" name="post_type" value="product">	
				<input type="hidden" name="page" value="wbte-products">
				<input type="hidden" name="view" value="upsell">
				<input type="hidden" name="categories" value="">
				<div id="cas-on-off-switch-div2" class="cas-on-off-switch" title="<?php esc_attr_e( 'Filter in table rows(left) or Search (database)', 'woo-bulk-categories' ); ?>">
					<input type="checkbox" class="cas-toggle-input" id="cas-search-toggle2" <?php echo esc_attr( strlen( $search_str2 ) > 0 ? 'checked="checked"' : '' ); ?>>
					<label class="cas-toggle-label" for="cas-search-toggle"></label>
				</div>
				<input type="hidden" name="s" value="<?php echo esc_attr( $search_str ); ?>">
				<input type="search" id="wbte_s2" onmouseover="this.focus();" name="s2" class="wbc-txt-search" value="<?php //esc_attr_e( $filter ); ?>" >
				<button type="submit" id="wbte_search_submit2" class="button"><i class="fas fa-search"></i> <?php esc_attr_e( 'Search', 'woo-bulk-categories' ); ?></button>
				
				</form>
			</div>
				
			<!-- Product Category Filter -->
			<div class="wbte-category-filter-style" style="display:flex;">
				<div style="flex-grow:5;">
				<form method="get" action="edit.php" id="cas-form-category2-filter" onsubmit="cas_upsell_products_get(this,'cat2','categories2');">
					<input type="hidden" name="post_type" value="product">	
					<input type="hidden" name="page" value="wbte-products">
					<input type="hidden" name="view" value="upsell">
					<input type="hidden" name="categories" value="">
					<input type="hidden" name="categories2" value="<?php echo esc_attr( $categories ); ?>">
					<?php self::wbte_create_category_dropdown( 'wbte-category-filter-upsell', 'cat2' ); ?>  
					<input type="hidden" name="s" value="<?php echo esc_attr( $search_str ); ?>">
					<button type="submit" class="button" title="<?php esc_attr_e( 'Get products from categories', 'woo-bulk-table-editor' ); ?>"> <i class="fa-solid fa-filter"></i> <?php esc_attr_e( 'Get', 'woo-bulk-table-editor' ); ?></button>
				</form>
				</div>
			</div>

			</div>

			<table id="wbte-up-cross-sell-table" class="wp-list-table widefat fixed striped">
				<thead>
					<tr>
						<th style="width:35px;"><input type="checkbox" onclick="wbteCheckAllRows('wbte-up-cross-sell-table', 'productUp', this);"> <i class="fa-solid fa-circle-question" title="<?php esc_attr_e( 'Click to select all / none', 'woo-bulk-table-editor' ); ?>"></i></th>
						<th style="width:60px;"><?php esc_attr_e( 'ID', 'woo-bulk-table-editor' ); ?></th>
						<th style="width:30px;"></th>
						<th><?php esc_attr_e( 'Name', 'woo-bulk-table-editor' ); ?> <i class="fa-solid fa-circle-question" title="<?php esc_attr_e( 'Click checkbox or in a row to select products', 'woo-bulk-table-editor' ); ?>"></i></th>
						<th><?php esc_attr_e( 'Price', 'woo-bulk-table-editor' ); ?></th>
					</tr>
				</thead>
				<tbody>
				<?php 
				if ( $products ) : 
					foreach ( $products as $product ) {
						?>
						<tr>
						<td><input type="checkbox" name="productUp" id="<?php echo esc_attr( $product->get_id() ); ?>"></td>
						<td><a href="<?php echo esc_url( get_admin_url() . 'post.php?post=' . $product->get_id() . '&action=edit' ); ?>" target="new"><?php echo esc_attr( $product->get_id() ); ?></a></td>
						<td>
						<?php
						if ( $product->get_image_id() ) {
							echo wp_kses( get_the_post_thumbnail( $product->get_id(), array( '28', '28' ), array( 'class' => 'wbte-thumb-main' ) ), 'post' );
						} else {
							?>
							<img height="28" src="<?php echo esc_attr( wc_placeholder_img_src() ); ?>" class="wbte-thumb-main wp-post-image">
							<?php
						}
						?>
						</td>
						<td>
							<a href="<?php echo esc_url( get_admin_url() . 'post.php?post=' . $product->get_id() . '&action=edit' ); ?>" target="new"><?php echo esc_attr( $product->get_name() ); ?></a><br/>
							<small><?php esc_attr_e( 'SKU:', 'woo-bulk-table-editor' ); ?> <?php echo esc_attr( $product->get_sku() ); ?></small>
							<?php 
							if ( class_exists( 'WC_Brands' ) ) {
								$brands = self::wbte_get_product_brands( $product->get_id(), 'name' );
								if ( ! empty( $brands ) ) {
									?>
										<br/><small><?php esc_attr_e( 'Brand:', 'woo-bulk-table-editor' ); ?> <?php echo esc_attr( $brands ); ?></small>
									<?php
								}
							}
							?>
						</td>
						<td>
							<?php echo wp_kses( ( $product->is_on_sale() ) ? '<s>' . wc_price( $product->get_regular_price() ) . '</s> ' . wc_price( $product->get_price() ) : wc_price( $product->get_price() ), 'post' ); ?>
						</td>
						</tr>
						<?php 
					}
					?>
				<?php else : ?>
					<tr><td colspan="5"><?php esc_attr_e( 'No products found! Select categories or search for products.', 'woo-bulk-table-editor' ); ?></td></tr>			
				<?php endif; ?>
				</tbody>
			</table>
			<div style="margin-top: 20px;">
			<button type="button" class="button" onclick="wbteLinkProducts('up');" title="<?php esc_attr_e( 'Upsell checked products', 'woo-bulk-table-editor' ); ?>"> <i class="fa-solid fa-arrow-up"></i> <?php esc_attr_e( 'Upsell', 'woo-bulk-table-editor' ); ?></button>
			<button type="button" class="button" onclick="wbteLinkProducts('cross');" title="<?php esc_attr_e( 'Cross-Sell checked products', 'woo-bulk-table-editor' ); ?>"> <i class="fa-solid fa-arrows-split-up-and-left"></i> <?php esc_attr_e( 'Cross-Sell', 'woo-bulk-table-editor' ); ?></button>
			<button type="button" class="button" onclick="wbteLinkProducts('up-cross');" title="<?php esc_attr_e( 'Upsell and Cross-Sell checked products', 'woo-bulk-table-editor' ); ?>"> <i class="fa-solid fa-arrows-up-down-left-right"></i> <?php esc_attr_e( 'Upsell and Cross-Sell', 'woo-bulk-table-editor' ); ?></button>
			</div>

		</div>
		<?php
	}
}
