<?php
/**
 * Bulk Table Editor
 *
 * @package BulkTableEditor/includes
 */

defined( 'ABSPATH' ) || exit;

require_once __DIR__ . '/class-wbte-functions.php';

/**
 * Class for admin tasks
 */
class Wbte_Woo {

	/**
	 * Action & Hooks
	 */
	public static function wbte_admin_hooks() {

		// Admin-specific hooks
		if ( is_admin() ) {
			add_action( 'admin_menu', array( __CLASS__, 'wbte_woocommerce_submenu_page' ) );
			add_action( 'admin_notices', array( __CLASS__, 'wbte_admin_activation_notice_success' ) );
			add_action( 'admin_enqueue_scripts', array( __CLASS__, 'wbte_header_scripts' ) );
			add_filter( 'admin_footer_text', array( __CLASS__, 'wbte_set_footer_text' ) );
			add_action( 'woocommerce_admin_field_button', array( __CLASS__, 'wbte_add_admin_field_select' ) );
			add_action( 'admin_head', array( __CLASS__, 'wbte_add_button_wbte' ) );
			add_action( 'admin_post_return_products_csv_file', array( __CLASS__, 'wbte_create_csv_file' ) );
			add_action( 'woocommerce_update_options_wbte', array( __CLASS__, 'wbte_set_new_settings_cron_job' ) );
		}

		// Backend attribute changes
		add_action( 'woocommerce_attribute_added', array( __CLASS__, 'wbte_delete_product_attributes_transient' ) );
		add_action( 'woocommerce_attribute_updated', array( __CLASS__, 'wbte_delete_product_attributes_transient' ) );
		add_action( 'woocommerce_attribute_deleted', array( __CLASS__, 'wbte_delete_product_attributes_transient' ) );

		// Variation cache clearing (admin-only)
		if ( is_admin() ) {
			add_action( 'woocommerce_product_object_updated_props', array( __CLASS__, 'wbte_clear_variation_cache' ), 10, 2 );
			add_action( 'woocommerce_new_product', array( __CLASS__, 'wbte_clear_variation_cache_on_new' ), 10, 1 );
			add_action( 'woocommerce_delete_product', array( __CLASS__, 'wbte_clear_variation_cache_on_delete' ), 10, 1 );
		}

		// Async actions for table operations (backend & frontend support)
		add_action( 'wbte_update_table_rows_data_async', array( __CLASS__, 'wbte_update_table_rows_data' ) );
		add_action( 'wbte_update_table_rows_ext_data_async', array( __CLASS__, 'wbte_update_table_rows_ext_data' ) );
		add_action( 'wbte_delete_table_rows_data_async', array( __CLASS__, 'wbte_delete_table_rows_data' ) );
		add_action( 'wbte_action_cyp_create_link_to_cyp', array( __CLASS__, 'wbte_create_link_to_cyp' ) );

		// Cron jobs and Action Scheduler setup
		if ( class_exists( 'ActionScheduler' ) ) {
			add_filter( 'action_scheduler_queue_runner_time_limit', array( __CLASS__, 'wbte_as_queue_runner_time_limit' ) );
			add_action( 'wbte_set_schedules', array( __CLASS__, 'wbte_add_custom_action_schedules' ) );
			add_action( 'wbte_start_stop_sales_hook', array( __CLASS__, 'wbte_run_start_stop_sales' ) );
			do_action( 'wbte_set_schedules' );
		} else {
			add_filter( 'cron_schedules', array( __CLASS__, 'wbte_add_custom_cron_interval' ) );
			add_action( 'wbte_reschedule_woocommerce_sales_event', array( __CLASS__, 'wbte_do_reschedule_woocommerce_sales_event' ) );
			do_action( 'wbte_reschedule_woocommerce_sales_event' );
		}

		// Dropdown customization (admin-only)
		if ( is_admin() ) {
			add_filter( 'wp_dropdown_cats', array( __CLASS__, 'wbte_add_multiple_select' ), 10, 2 );
		}

		// AJAX handlers 
		add_action( 'wp_ajax_wbte_update_table_rows_data', array( __CLASS__, 'wbte_update_table_rows_ajax_handler' ) );
		add_action( 'wp_ajax_nopriv_wbte_update_table_rows_data', array( __CLASS__, 'wbte_update_table_rows_ajax_handler' ) );
		add_action( 'wp_ajax_wbte_update_table_rows_ext_data', array( __CLASS__, 'wbte_update_table_rows_ext_ajax_handler' ) );
		add_action( 'wp_ajax_nopriv_wbte_update_table_rows_ext_data', array( __CLASS__, 'wbte_update_table_rows_ext_ajax_handler' ) );
		add_action( 'wp_ajax_wbte_delete_table_rows_data', array( __CLASS__, 'wbte_delete_table_rows_ajax_handler' ) );
		add_action( 'wp_ajax_nopriv_wbte_delete_table_rows_data', array( __CLASS__, 'wbte_delete_table_rows_ajax_handler' ) );
		add_action( 'wp_ajax_wbte_update_product_brands_data', array( __CLASS__, 'wbte_update_product_brands_data_callback' ) );
		add_action( 'wp_ajax_nopriv_wbte_update_product_brands_data', array( __CLASS__, 'wbte_update_product_brands_data_callback' ) );
		add_action( 'wp_ajax_wbte_update_category_image_data', array( __CLASS__, 'wbte_update_category_image_data_callback' ) );
		add_action( 'wp_ajax_nopriv_wbte_update_category_image_data', array( __CLASS__, 'wbte_update_category_image_data_callback' ) );

		// Attributes save
		add_action( 'wp_ajax_wbte_save_attributes_and_custom', array( __CLASS__, 'wbte_save_attributes_and_custom_callback' ) );
		add_action( 'wp_ajax_nopriv_wbte_save_attributes_and_custom', array( __CLASS__, 'wbte_save_attributes_and_custom_callback' ) );
		add_filter( 'woocommerce_attribute_label', array( __CLASS__, 'wbte_customize_local_attribute_label' ), 10, 2 );

		// Upsells
		add_action( 'wp_ajax_wbte_upsell_set_linked_products', array( 'WbteFunctions', 'wbte_upsell_set_linked_products_callback' ) );
		add_action( 'wp_ajax_nopriv_wbte_upsell_set_linked_products', array( 'WbteFunctions', 'wbte_upsell_set_linked_products_callback' ) );
		add_action( 'wp_ajax_wbte_upsell_remove_linked_product', array( 'WbteFunctions', 'wbte_upsell_remove_linked_product_callback' ) );
		add_action( 'wp_ajax_nopriv_wbte_upsell_remove_linked_product', array( 'WbteFunctions', 'wbte_upsell_remove_linked_product_callback' ) );

		
		// Settings and translations
		add_filter( 'woocommerce_get_settings_pages', array( __CLASS__, 'wbte_add_settings' ) );
		self::wbte_load_translations();
		
	}

	/**
	 * Delete product attributes transient if CRUD operations
	 */
	public static function wbte_delete_product_attributes_transient() {
		$transient_key = 'wbte_all_product_attributes';
		delete_transient( $transient_key );
	}

	/**
	 * Set up and use Action Scheduler
	 */
	public static function wbte_add_custom_action_schedules() {
		
		$options = get_option( 'wbte_options' );
		wp_clear_scheduled_hook( 'wbte_sale_cron_job_interval' );

		if ( ! empty( $options['wbte_use_sales_cron'] ) && 'yes' === $options['wbte_use_sales_cron'] ) {
			$interval = ( ! empty( $options['wbte_sale_cron_interval'] ) ) ? absint( $options['wbte_sale_cron_interval'] ) : HOUR_IN_SECONDS;
			if ( ! as_next_scheduled_action( 'wbte_start_stop_sales_hook' ) ) {
				as_schedule_recurring_action( time(), $interval, 'wbte_start_stop_sales_hook', array() );
			}
		} else {
			as_unschedule_all_actions( 'wbte_start_stop_sales_hook' );
		}

	}

	/**
	 * Start and stop WooCommerce sales
	 */
	public static function wbte_run_start_stop_sales() {
		//Run WooCommerce scheduled sales function
		wc_scheduled_sales();
	}

	/**
	 * Update the cron job if action scheduler
	 */
	public static function wbte_set_new_settings_cron_job() {
			
		if ( class_exists( 'ActionScheduler' ) ) {
			$hook = 'wbte_start_stop_sales_hook';
			as_unschedule_all_actions( $hook );
			//Reschedule
			self::wbte_add_custom_action_schedules();
		}
		
	}

	/**
	 * Increase time limit for action scheduler
	 */
	public static function wbte_as_queue_runner_time_limit( $time_limit ) {
		return 60;
	}

	/**
	 * Create a custom cron interval
	 */
	public static function wbte_add_custom_cron_interval( $schedules ) {
		
		$options = get_option( 'wbte_options' );

		if ( ! empty( $options['wbte_use_sales_cron'] ) && 'yes' === $options['wbte_use_sales_cron'] ) {
			$schedules['wbte_sale_cron_job_interval'] = array(
				'interval' => ( ! empty( $options['wbte_sale_cron_interval'] ) ) ? absint( $options['wbte_sale_cron_interval'] ) : HOUR_IN_SECONDS,
				'display'  => __( 'Bulk Table Editor - Sales Interval', 'woo-bulk-table-editor' ),
			);
		}

		return $schedules;
	}

	/**
	 * Reschedule WooCommerce sales interval
	 */
	public static function wbte_do_reschedule_woocommerce_sales_event() {

		$options = get_option( 'wbte_options' );

		// Check transient
		if ( get_transient( 'wbte_cron_setup_done' ) !== 'yes' ) {

			// Ensure the default WooCommerce scheduled sales event is set
			if ( ! wp_next_scheduled( 'woocommerce_scheduled_sales' ) ) {
				wp_schedule_event( time(), 'daily', 'woocommerce_scheduled_sales' );
			}

			// Run again after 24 hours to make sure we have it
			set_transient( 'wbte_cron_setup_done', 'yes', 24 * HOUR_IN_SECONDS );
		}

		if ( ! empty( $options['wbte_use_sales_cron'] ) && 'yes' === $options['wbte_use_sales_cron'] ) {
			
			$hook = 'woocommerce_scheduled_sales';

			// Clear the existing scheduled sale event
			$timestamp = wp_next_scheduled( $hook );
			if ( $timestamp ) {
				wp_unschedule_event( $timestamp, $hook );
			}

			// Schedule the new event with the custom interval
			if ( ! wp_next_scheduled( $hook ) ) {
				wp_schedule_event( time(), 'wbte_sale_cron_job_interval', $hook );
			}
		}
	}
	
	/**
	 * Create submenu in products
	 */
	public static function wbte_woocommerce_submenu_page() {

		add_submenu_page(
			'edit.php?post_type=product',
			__( 'Bulk Table Editor', 'woo-bulk-table-editor' ),
			__( 'Bulk Table Editor', 'woo-bulk-table-editor' ),
			'manage_woocommerce',
			'wbte-products',
			array( __CLASS__, 'wbte_products_page' ),
			1
		);

	}

	/**
	 * Get Bulk Table Editor main page
	 */
	public static function wbte_products_page() {

		if ( class_exists( 'WooCommerce' ) ) {
			require_once __DIR__ . '/class-wbte-templates.php';
			$editor = new WbteTemplates();
			$editor->wbte_load(); 
		}

	}

	/**
	 * Add settings and language support
	 */
	public static function wbte_add_settings( $settings ) {

		$settings[] = include_once dirname( __FILE__ ) . '../../includes/class-wbte-settings.php';
		return $settings;

	}

	/**
	 * Add translations
	 */
	public static function wbte_load_translations() {

		load_plugin_textdomain( 'woo-bulk-table-editor', false, dirname( dirname( plugin_basename( __FILE__ ) ) ) . '/languages' );
	
	}

	/**
	 * On activation notice
	 */
	public static function wbte_admin_activation_notice_success() {

		$allowed_tags = array(
			'a' => array(
				'class'  => array(),
				'href'   => array(),
				'target' => array(),
				'title'  => array(),
			),
		);
		/* translators: %s: url for documentation */
		$read_doc = __( 'Read the <a href="%1$s" target="_blank"> extension documentation </a> for more information.', 'woo-bulk-table-editor' );
		$out_str  = sprintf( $read_doc, esc_url( 'https://docs.woocommerce.com/document/woocommerce-bulk-table-editor' ) );

		if ( get_transient( 'wbte-admin-notice-activated' ) ) {
			?>
			<div class="updated woocommerce-message">
				<a class="woocommerce-message-close notice-dismiss" href="<?php echo esc_url( wp_nonce_url( add_query_arg( 'wc-hide-notice', 'wbte_admin_activation_notice_success' ), 'woocommerce_hide_notices_nonce', '_wc_notice_nonce' ) ); ?>"><?php esc_html_e( 'Dismiss', 'woo-bulk-table-editor' ); ?></a>
				<p>
					<?php esc_html_e( 'Thank you for installing Bulk Table Editor for WooCommerce. You can now start to bulk update products. ', 'woo-bulk-table-editor' ); ?>
					<?php echo wp_kses( $out_str, $allowed_tags ); ?>
				</p>
				<p class="submit">
					<a href="<?php echo esc_url( admin_url( 'edit.php?post_type=product&page=wbte-products' ) ); ?>" class="button-primary"><?php esc_html_e( 'Start bulk editing products', 'woo-bulk-table-editor' ); ?></a>
					<a href="<?php echo esc_url( admin_url( 'admin.php?page=wc-settings&tab=wbte' ) ); ?>" class="button-secondary"><?php esc_html_e( 'Settings', 'woo-bulk-table-editor' ); ?></a>
				</p>
			</div>

			<?php
			delete_transient( 'wbte-admin-notice-activated' );
		}
	}
	
	/**
	 * Add scripts and style
	 *
	 * @param var $hook object.
	 */
	public static function wbte_header_scripts( $hook ) {

		if ( 'product_page_wbte-products' !== $hook ) {
			return;
		}

		wp_register_script( 'wbte_jquery-ui', 'https://code.jquery.com/ui/1.13.1/jquery-ui.js', array( 'jquery' ), '1.13.1', true );
		wp_enqueue_script( 'wbte_jquery-ui' );

		wp_register_script( 'cas_multiselect', plugins_url( '../assets/js/multiselect/jquery.multiselect.js', __FILE__ ), array( 'jquery' ), CAS_BULK_TABLE_EDITOR_VS, true );
		wp_enqueue_script( 'cas_multiselect' );

		wp_register_script( 'cas_multiselect_filter', plugins_url( '../assets/js/multiselect/jquery.multiselect.filter.js', __FILE__ ), array( 'jquery' ), CAS_BULK_TABLE_EDITOR_VS, true );
		wp_enqueue_script( 'cas_multiselect_filter' );

		if ( 'true' === CAS_BULK_TABLE_EDITOR_PROD ) {
			wp_register_script( 'wbte_tablecalc', plugins_url( '../assets/js/jquery.tablecalc.min.js', __FILE__ ), array( 'jquery' ), CAS_BULK_TABLE_EDITOR_VS, false );
			wp_register_script( 'wbte_scripts', plugins_url( '../assets/js/wbte-scripts.min.js', __FILE__ ), array( 'jquery' ), CAS_BULK_TABLE_EDITOR_VS, true );
		} else {
			wp_register_script( 'wbte_tablecalc', plugins_url( '../assets/js/jquery.tablecalc.js', __FILE__ ), array( 'jquery' ), CAS_BULK_TABLE_EDITOR_VS, false );
			wp_register_script( 'wbte_scripts', plugins_url( '../assets/js/wbte-scripts.js', __FILE__ ), array( 'jquery' ), CAS_BULK_TABLE_EDITOR_VS, true );
		}

		wp_enqueue_script( 'wbte_tablecalc' );
			
		wp_register_script( 'lozad', 'https://cdn.jsdelivr.net/npm/lozad/dist/lozad.min.js', array(), '1', false );
		wp_enqueue_script( 'lozad' );

		wp_register_style( 'fonta', 'https://use.fontawesome.com/releases/v6.4.2/css/all.css', array(), '6.4.2' );
		wp_enqueue_style( 'fonta' );

		wp_register_style( 'jquery-ui', 'https://code.jquery.com/ui/1.13.1/themes/base/jquery-ui.css', array(), '1.13.1', 'all' );
		wp_enqueue_style( 'jquery-ui' );

		wp_register_style( 'cas_multiselect_css', plugins_url( '../assets/css/multiselect/jquery.multiselect.css', __FILE__ ), array(), CAS_BULK_TABLE_EDITOR_VS );
		wp_enqueue_style( 'cas_multiselect_css' );

		wp_register_style( 'cas_multiselect_filter_css', plugins_url( '../assets/css/multiselect/jquery.multiselect.filter.css', __FILE__ ), array(), CAS_BULK_TABLE_EDITOR_VS );
		wp_enqueue_style( 'cas_multiselect_filter_css' );

		if ( 'true' === CAS_BULK_TABLE_EDITOR_PROD ) {
			wp_register_style( 'wbte_css', plugins_url( '../assets/css/wbte.min.css', __FILE__ ), array(), CAS_BULK_TABLE_EDITOR_VS );
		} else {
			wp_register_style( 'wbte_css', plugins_url( '../assets/css/wbte.css', __FILE__ ), array(), CAS_BULK_TABLE_EDITOR_VS );
		}
		wp_enqueue_style( 'wbte_css' );

		wp_register_style( 'wbte_dark_css', plugins_url( '../assets/css/wbte-dark.css', __FILE__ ), array(), CAS_BULK_TABLE_EDITOR_VS );
		wp_enqueue_style( 'wbte_dark_css' );

		$options = get_option( 'wbte_options' );
		// Localize the script with new data
		$translation = array(
			'media_lib_title'     => __( 'Product image', 'woo-bulk-table-editor' ),
			'media_lib_button'    => __( 'Select image', 'woo-bulk-table-editor' ),
			'select_category'     => __( 'Select Category', 'woo-bulk-table-editor' ),
			'sku_count'           => ( $options['wbte_sku_count'] ) ? $options['wbte_sku_count'] : 3,
			'sku_delimiter'       => ( $options['wbte_sku_delimiter'] ) ? $options['wbte_sku_delimiter'] : '-',
			'products'		      => __( 'Products', 'woo-bulk-table-editor' ),
			'variations'	      => __( 'Variations', 'woo-bulk-table-editor' ),
			'disable_description' => ( $options['wbte_disable_description'] ) ? $options['wbte_disable_description'] : 'no',
			'js_sale_filter'      => ! empty( $options[ 'wbte_table_sale_filter' ] ) ? $options[ 'wbte_table_sale_filter' ] : 'no',
			'js_show_sku'         => ! empty( $options[ 'wbte_use_sku_main_page' ] ) ? $options[ 'wbte_use_sku_main_page' ] : 'no',
			'js_show_vendors'     => ! empty( $options[ 'wbte_vendor_integration' ] ) ? $options[ 'wbte_vendor_integration' ] : 'no',
			'js_custom_active'    => ! empty( $options[ 'wbte_custom_price_1_visible' ] ) ? $options[ 'wbte_custom_price_1_visible' ] : 'no',
			'brand_filter'        => class_exists( 'WC_Brands' ) ? 'yes' : 'no',
			'brand_active'        => ! empty( $options[ 'wbte_use_brands' ] ) ? $options[ 'wbte_use_brands' ] : 'no',
			'custom_fields'       => Wbte_Custom_Fields::wbte_get_custom_fields_array(),
			'use_dark_mode'       => ! empty( $options[ 'wbte_use_dark_mode' ] ) ? $options[ 'wbte_use_dark_mode' ] : 'no',
		);
		wp_localize_script( 'wbte_scripts', 'wbte_object', $translation );
		wp_enqueue_script( 'wbte_scripts' );

		//Media
		wp_enqueue_media();
		wp_enqueue_editor();
	}

	/**
	 * Set footer text.
	 */
	public static function wbte_set_footer_text( $text ) {

		$page = filter_input( 1, 'page', FILTER_SANITIZE_FULL_SPECIAL_CHARS );

		if ( 'wbte-products' === $page ) {
			?>
			<span> </span>
			<?php

		} else {

			return $text;

		}

	}

	/**
	 * Add admin select category
	 *
	 * @param var $value object.
	 */
	public static function wbte_add_admin_field_select( $value ) {

		$wbte_options   = get_option( 'wbte_options' );
		$wbte_functions = new WbteFunctions();

		?>
			<tr valign="top">
				<th scope="row">
					<label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_attr( $value['title'] ); ?></label>
					<span class="woocommerce-help-tip" data-tip="<?php echo esc_attr( $value['desc_tip'] ); ?>"></span>
				</th>
				<td>
					<?php $wbte_functions->wbte_settings_get_categories_select( $wbte_options ); ?>
				</td>
			</tr>
		<?php
	}

	/**
	 * Add button to edit product
	 */
	public static function wbte_add_button_wbte() {
		
		global $current_screen;

		if ( 'product' !== $current_screen->id ) {
			return;
		}

		?>
		<script>
		jQuery( function() {
			jQuery(' <a href="<?php echo esc_url( admin_url( 'edit.php?post_type=product&page=wbte-products' ) ); ?>" class="page-title-action"><?php esc_attr_e( 'Bulk Table Editor', 'woo-bulk-table-editor' ); ?></a>').insertBefore('.wp-header-end');
		});
		</script>
		<?php
		
	}

	/** 
	 * If Calculate your price / Cost & Reports / Bulk Category Editor is present create link
	 */
	public static function wbte_create_link_to_cyp() {

		if ( is_plugin_active( 'woo-calculate-your-price/woo-calculate-your-price.php' ) ) {
			
			global $cyp_version;
			$p_cat = filter_input( 1, 'product_cat', FILTER_SANITIZE_FULL_SPECIAL_CHARS );

			if ( version_compare( $cyp_version, '3.0.1', '>=' ) >= 0 ) {
				?>
				<a type="button" href="<?php echo esc_url( admin_url( 'admin.php?page=crw&product_cat=' . $p_cat ) ); ?>" class="button" id="wbte" style="margin-left:4px;">
					<i class="fas fa-chart-line"></i> <?php esc_attr_e( 'Cost & Reports', 'woo-bulk-table-editor' ); ?></a>
				<?php
			} else {
				?>
				<a type="button" href="<?php echo esc_url( admin_url( 'edit.php?post_type=product&page=cyp&product_cat=' . $p_cat ) ); ?>" class="button" id="wbte" style="margin-left:4px;">
					<i class="fas fa-chart-line"></i> <?php esc_attr_e( 'Calculate Your Price', 'woo-bulk-table-editor' ); ?></a>
				<?php
			}

		}

		if ( is_plugin_active( 'woo-bulk-category-editor/woo-bulk-category-editor.php' ) ) {
			?>
				<a type="button" href="<?php echo esc_url( admin_url( 'admin.php?page=wbc' ) ); ?>" class="button" id="wbc" style="margin-left:4px;">
				<i class="fas fa-th-list"></i> <?php esc_attr_e( 'Bulk Category Editor', 'woo-bulk-table-editor' ); ?></a>
			<?php
		}

	}

	/**
	 * Update product
	 */
	public static function wbte_update_table_rows_ajax_handler() {
	
		check_ajax_referer( 'footer_id', 'nonce' );
		$data = wp_unslash( $_POST );

		if ( apply_filters( 'wbte_update_table_rows_data_async', true ) ) {
			WC()->queue()->add( 'wbte_update_table_rows_data_async', $data, 'wbte_work_queue' );
		} else {
			self::wbte_update_table_rows_data( $data );
		}

		//Refresh data
		delete_transient( 'cas_json_products' );

		wp_die();

	}

	/**
	 * Update product ext table
	 */
	public static function wbte_update_table_rows_ext_ajax_handler() {
		
		check_ajax_referer( 'footer_id_ext', 'nonce' );
		$data = wp_unslash( $_POST );

		if ( apply_filters( 'wbte_update_table_rows_ext_data_async', true ) ) {
			WC()->queue()->add( 'wbte_update_table_rows_ext_data_async', $data, 'wbte_work_queue' );
		} else {
			self::wbte_update_table_rows_ext_data( $data );
		}

		//Refresh data
		delete_transient( 'cas_json_products' );

		wp_die();
	}

	/**
	 * Remove brand from a product
	 */
	public static function wbte_update_product_brands_data_callback() {
		
		check_ajax_referer( 'footer_id', 'nonce' );

		$data      = wp_unslash($_POST);
		$productId = isset( $data['product_id'] ) ? intval( $data['product_id'] ) : 0;
		$termId    = isset( $data['term_id'] ) ? intval( $data['term_id'] ) : 0;
	
		if ( $productId > 0 && $termId > 0 ) {
			// Remove the term from the product
			$result = wp_remove_object_terms( $productId, $termId, 'product_brand' );
	
			if ( is_wp_error( $result ) ) {
				// Handle error
				wp_send_json_error( $result->get_error_message(), 400 );
			} else {
				// Success
				wp_send_json_success( __( 'Brand removed successfully.', 'woo-bulk-table-editor' ), 200 );
			}
		} else {
			// Missing or invalid data
			wp_send_json_error( __( 'Missing or invalid product ID or brand ID.', 'woo-bulk-table-editor' ), 400 );
		}

		wp_die();
	}

	/**
	 * Update Categories and Images
	 */
	public static function wbte_update_category_image_data_callback() {
		check_ajax_referer( 'wbte_id', 'nonce' );

		$data = wp_unslash($_POST);
		$wbte = new WbteFunctions();
		$rows = ( isset( $data['rows'] ) ) ? (array) $data['rows'] : array();
		
		foreach ( $rows as $row ) {
			$wbte->wbte_update_categories_images( $row );
		}

		wp_send_json_success( __( 'Update Success', 'woo-bulk-table-editor' ), 200 );
		wp_die();
	}

	/**
	 * Update Attributes and custom fields
	 */
	public static function wbte_save_attributes_and_custom_callback() {
		if ( ! check_ajax_referer( 'wbte_id', 'nonce' ) ) {
			error_log( 'Nonce is not valid - in: wbte_save_attributes_and_custom' );
			wp_die();
		}

		$data      = wp_unslash($_POST);
		$functions = new WbteFunctions();
		$rows      = ( isset( $data['rows'] ) ) ? (array) $data['rows'] : array();
		
		foreach ( $rows as $row ) {
			$functions->wbte_save_attributes_and_custom( $row );
		}

		wp_send_json_success( __( 'Update Success', 'woo-bulk-table-editor' ), 200 );
		wp_die();
	}

	/**
	 * Customize the display name of local product attributes.
	 *
	 * @param string $label The original attribute label.
	 * @param string $name The attribute name.
	 * @return string The customized attribute label.
	 */
	public static function wbte_customize_local_attribute_label( $label, $name ) {
		// Check if the attribute is local (doesn't start with 'pa_')
		if ( strpos( $name, 'pa_' ) !== 0 ) {
			// Convert the attribute name to title case and replace hyphens with spaces
			$label = ucwords( str_replace( '-', ' ', $name ) );
		}
		return $label;
	}

	/**
	 * Update products
	 * 
	 * @param var $data data.
	 */
	public static function wbte_update_table_rows_data( $data ) {

		$wbte = new WbteFunctions();
		$rows = ( isset( $data['rows'] ) ) ? (array) $data['rows'] : array();
		
		foreach ( $rows as $row ) {
			$wbte->wbte_update_product( $row );
		}
		
	}

	/**
	 * Update products ext table
	 * 
	 * @param var $data data.
	 */
	public static function wbte_update_table_rows_ext_data( $data ) {

		$wbte = new WbteFunctions();
		$rows = ( isset( $data['rows'] ) ) ? (array) $data['rows'] : array();

		foreach ( $rows as $row ) {
			$wbte->wbte_update_product_ext( $row );
		}	
		
	}

	/**
	 * Delete product (ajax)
	 */
	public static function wbte_delete_table_rows_ajax_handler() {
		
		check_ajax_referer( 'footer_id', 'nonce' );
		$data = wp_unslash( $_POST );

		if ( apply_filters( 'wbte_delete_table_rows_data_async', true ) ) {
			WC()->queue()->add( 'wbte_delete_table_rows_data_async', $data, 'wbte_work_queue' );
		} else {
			self::wbte_delete_table_rows_data( $data );
		}

		//Refresh data
		delete_transient( 'cas_json_products' );

		wp_die();
	}

	/**
	 * Delete products
	 * 
	 * @param var $data data.
	 */
	public static function wbte_delete_table_rows_data( $data ) {

		$wbte = new WbteFunctions();
		$rows = ( isset( $data['rows'] ) ) ? (array) $data['rows'] : array();

		foreach ( $rows as $row ) {
			$wbte->wbte_move_to_trash( $row );
		}

	}

	/**
	 * Create csv file
	 */
	public static function wbte_create_csv_file() {

		$functions = new WbteFunctions();
		$functions->wbte_create_csv_export_file();

	}

	/**
	 * Add multiple select
	 */
	public static function wbte_add_multiple_select( $output, $args ) {
		
		if ( isset( $args['wbte_multi_select'] ) && $args['wbte_multi_select'] ) {

			$output   = preg_replace( '/^<select/i', '<select multiple ', $output );
			$output   = str_replace( "name='{$args['name']}'", "name='{$args['name']}[]'", $output );
			$selected = is_array($args['selected']) ? $args['selected'] : explode( ',', $args['selected'] );
			
			foreach ( array_map( 'trim', $selected ) as $value ) {
				$output = str_replace( "value=\"{$value}\"", "value=\"{$value}\" selected", $output );
			}
		}
	
		return $output;
	}

	/**
	 * Clear variation cache when a product is updated.
	 *
	 * @param WC_Product $product The product object.
	 * @param array      $updated_props The properties that were updated.
	 */
	public static function wbte_clear_variation_cache( $product, $updated_props ) {
		if ( $product->is_type('variable') ) {
			self::wbte_clear_variation_transient( $product->get_id() );
		} elseif ( $product->is_type('product_variation') ) {
			self::wbte_clear_variation_transient( $product->get_parent_id() );
		}
	}

	/**
	 * Clear variation cache when a new product is created.
	 *
	 * @param int $product_id The product ID.
	 */
	public static function wbte_clear_variation_cache_on_new( $product_id ) {
		$product = wc_get_product( $product_id );
		if ( $product && $product->is_type('variable') ) {
			self::wbte_clear_variation_transient( $product->get_id() );
		}
	}

	/**
	 * Clear variation cache when a product is deleted.
	 *
	 * @param int $product_id The product ID.
	 */
	public static function wbte_clear_variation_cache_on_delete( $product_id ) {
		$product = wc_get_product( $product_id );
		if ( $product && $product->is_type('variable') ) {
			self::wbte_clear_variation_transient( $product->get_id() );
		}
	}

	/**
	 * Helper function to delete variation cache for a product ID.
	 *
	 * @param int $product_id The product ID.
	 */
	public static function wbte_clear_variation_transient( $product_id ) {
		$cache_key = 'wbte_variations_cache_' . $product_id;
		delete_transient( $cache_key );
	}


}
Wbte_Woo::wbte_admin_hooks();

