<?php
/**
 * Bulk Table Editor
 *
 * @package BulkTableEditor
 *
 * Plugin Name: Bulk Table Editor for WooCommerce
 * Plugin URI: https://woocommerce.com/products/bulk-table-editor-for-woocommerce/
 * Description: Easily bulk edit WooCommerce product prices, stock, sales, SKUs, attributes, images, categories, tags, and more in a fast and intuitive table interface.
 * Version: 2.5.1
 * Author: Consortia
 * Author URI: http://www.consortia.no/en/
 * Text Domain: woo-bulk-table-editor
 * Domain Path: /languages
 *
 * Tested up to: 6.3.2
 * WC requires at least: 3.3
 * WC tested up to: 9.6.0
 *
 * Copyright: © 2018-2025 Consortia AS.
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 * Woo: 4703437:2514ffe2d12bf99178aacbc3755dc518

 */

const CAS_BULK_TABLE_EDITOR_VS   = '2.5.1';
const CAS_BULK_TABLE_EDITOR_PROD = 'true';

add_action( 'before_woocommerce_init', function() {
	if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
		\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
	}
} );

register_activation_hook( __FILE__, 'wbte_install' );
register_deactivation_hook( __FILE__, 'wbte_deactivate' );

require_once dirname( __FILE__ ) . '/includes/class-wbte-woo.php';

/**
 * Install
 *
 * @return void
 */
function wbte_install() {

	global $wp_version;

	if ( version_compare( $wp_version, '4.1', '<' ) ) {
		wp_die( 'This plugin require WordPress 4.1 or higher.' );
	}

	$wbte_options_arr = array(
		'wbte_posts_per_page'             => '',
		'wbte_product_cat'                => '',
		'wbte_no_stock_management'        => '',
		'wbte_no_autofocus'               => '',
		'wbte_custom_price_1'             => '',
		'wbte_custom_price_1_header'      => '',
		'wbte_custom_price_1_visible'     => '',
		'wbte_custom_price_1_normal_calc' => '',
		'wbte_date_format_autofocus'      => '',
		'wbte_use_sku_main_page'          => '',
		'wbte_vendor_integration'         => '',
		'wbte_sku_count'                  => '',
		'wbte_product_status'             => '',
		'wbte_sku_delimiter'              => '',
		'wbte_integration'                => '',
		'wbte_disable_description'        => '',
		'wbte_table_sale_filter'          => '',
		'wbte_hide_edit_options'          => '',
		'wbte_product_orderby'            => '',
		'wbte_product_order'              => '',
		'wbte_hide_range_filter'          => '',
		'wbte_use_sales_cron'             => '',
		'wbte_sale_cron_interval'         => '',
		'wbte_categories_all'             => '',
		'wbte_categories_orderby'         => '',
		'wbte_use_brands'                 => '',
		'wbte_inlcude_cat_in_search'      => '',
		'wbte_custom_field_1_name'        => '',
		'wbte_custom_field_1_type'        => '',
		'wbte_custom_field_1_slug'        => '',
		'wbte_use_custom_field_1'         => '',
		'wbte_custom_field_2_name'        => '',
		'wbte_custom_field_2_type'        => '',
		'wbte_custom_field_2_slug'        => '',
		'wbte_use_custom_field_2'         => '',
		'wbte_custom_field_3_name'        => '',
		'wbte_custom_field_3_type'        => '',
		'wbte_custom_field_3_slug'        => '',
		'wbte_use_custom_field_3'         => '',
		'wbte_custom_field_4_name'        => '',
		'wbte_custom_field_4_type'        => '',
		'wbte_custom_field_4_slug'        => '',
		'wbte_use_custom_field_4'         => '',
		'wbte_custom_field_5_name'        => '',
		'wbte_custom_field_5_type'        => '',
		'wbte_custom_field_5_slug'        => '',
		'wbte_use_custom_field_5'         => '',
		'wbte_use_dark_mode'		      => '',
	);
	
	update_option( 'wbte_options', $wbte_options_arr );

	set_transient( 'wbte-admin-notice-activated', true );
	delete_transient( 'wbte_cron_setup_done' );

	flush_rewrite_rules();

}

/**
 * Deactivate
 *
 * @return void
 */
function wbte_deactivate() {

	flush_rewrite_rules();
	
	wp_clear_scheduled_hook( 'wbte_sale_cron_job_interval' );
	
	if ( class_exists( 'ActionScheduler' ) ) {
		as_unschedule_all_actions( 'wbte_start_stop_sales_hook' );
	}

}

